package ics.uci.edu.Calico.Readers;


import ics.uci.edu.Calico.models.Snapshot;

import javax.swing.*;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Date;

public class BarCanvas extends JPanel{
	private int maxX;
	private int maxY;
	private Color backgroundColor;
	private Color moveLineColor = Color.gray;
	private Color viewIndicatorColor = Color.pink;
	private SnapshotHistory history;
	private FilmstripReader reader;
	private Vector gridLocationHistories;
	
	private Date lastPlacedTimestamp;
	private int currentX;
	private int currentLocation;
	private int locationLinkerHeight;
	private int viewIndicatorX;
	
	private static final int MAX_HEIGHT = 40;
	private static final int MIN_HEIGHT = 4;
	private static final int BUFFER = 10;
	private static final int BAR_WIDTH = 2;
	private static final int MOVE_LINE_WIDTH = 1;
	private static final int MOVE_LINE_HEIGHT = 45;
	private static final long MAX_GAP_MILLIS = 6000;
	private static final int VIEW_INDICATOR_HEIGHT = 45;
	
	public BarCanvas(FilmstripReader newReader, SnapshotHistory newHistory){
		backgroundColor = new Color((float)0.95, (float)0.95, (float)1.0);
		reader = newReader;
		history = newHistory;
		gridLocationHistories = new Vector();
		locationLinkerHeight = 2;
		viewIndicatorX = -1;
		
		lastPlacedTimestamp = new Date();
		addMouseListener(new BarClickListener());
	}
	
	public int getWidth(){
		return BUFFER * 2 + BAR_WIDTH * history.getSize();
	}
	
	public void paintComponent(Graphics g){		
		super.paintComponent(g);
		setBackground(backgroundColor);
		Snapshot temp;
		int height;
		currentX = BUFFER;
		currentLocation = 0;
		
		gridLocationHistories.clear();
		locationLinkerHeight = 2;
		
		for(int i = 0; i < history.getSize(); i++){
			temp = history.getSnapshotAt(i);
			
			//Draw grid movement lines
			if( temp.getGridCell() != currentLocation ){
				g.setColor(moveLineColor);
				g.fillRect(currentX, BUFFER, MOVE_LINE_WIDTH, MOVE_LINE_HEIGHT);
				currentX += MOVE_LINE_WIDTH;
				currentLocation = temp.getGridCell();
			
			//While we're in here, add this move to our list of locationHistories
				GridLocationHistory tempLocation = new GridLocationHistory(-1);
				boolean locationFound = false;
				
				//Search the locationHistories to see if we've been here before
				//If so, add this xcoordinate to that location
				for(int j = 0; j < gridLocationHistories.size() && !locationFound; j++){
					tempLocation = (GridLocationHistory)gridLocationHistories.elementAt(j);
					
					if(tempLocation.getLocation() == currentLocation){
						tempLocation.addXCoordinate(currentX);
						locationFound = true;
						
						//If this is the second one added, lets give it a proper height
						if(tempLocation.coordinateCount() == 2){
							
							tempLocation.setHeight(locationLinkerHeight);
							locationLinkerHeight += 2;
							if(locationLinkerHeight >= BUFFER){
								locationLinkerHeight = 2;
							}
						}
					}
				}
				
				//If not, add a new locationHistory to the histories
				if(!locationFound){
					GridLocationHistory tempHistory	= new GridLocationHistory(currentLocation);
					tempHistory.addXCoordinate(currentX);
					gridLocationHistories.addElement(tempHistory);
				}
				  
			}			
			
			//Action bars
			//Handle the height
			else{
				
				if(i == 0){
					lastPlacedTimestamp = temp.getTimestamp();
					height = MAX_HEIGHT;}
				else{
					height = getHeightFromTimestamp(temp);}
				
				if(temp.getType() != 0){
				g.setColor(getTypeColor(temp.getType()));
				g.fillRect(currentX, BUFFER, BAR_WIDTH, height);
				currentX += BAR_WIDTH;
				}
			}
			
			//View Indicator
			if(viewIndicatorX > -1){
				g.setColor(viewIndicatorColor);
				g.drawLine(viewIndicatorX, BUFFER - 5, viewIndicatorX, BUFFER + VIEW_INDICATOR_HEIGHT);
				g.fillRect(viewIndicatorX - 1, BUFFER + VIEW_INDICATOR_HEIGHT - 1, 3, 3);
			}
		}
		
		//Once we've gone through them all, we have our completed list of histories
		//Draw the connector lines
		g.setColor(Color.GRAY);
		
		//For each history
		for(int i = 0; i < gridLocationHistories.size(); i++){
			GridLocationHistory tempHistory = (GridLocationHistory)gridLocationHistories.elementAt(i);
			Vector tempXs = tempHistory.getXCoordinates();
			
			//If there's only one in this one, no need to draw it
			if(tempHistory.coordinateCount() > 1){

				int tempHeight = tempHistory.getHeight();
				
				//For each X location in that history, draw a vertial line
				for(int j = 0; j < tempXs.size(); j++){
					int tempX = ((Integer)tempXs.elementAt(j)).intValue();
					g.drawLine(tempX - 1, BUFFER, tempX - 1, BUFFER - tempHeight);
				}
				
				//Draw the vertical line for this history
				g.drawLine(((Integer)tempXs.firstElement()).intValue() - 1, BUFFER - tempHeight,
						((Integer)tempXs.lastElement()).intValue() - 1, BUFFER - tempHeight);
			}
		}
	}
	
	//Returns a height between MIN_HEIGHT and MAX_HEIGHT, based on the difference between
	//this snapshot and the last, relative to the maxiumum such gap (MAX_GAP_MILLIS)
	private int getHeightFromTimestamp(Snapshot s){
		Long newTime = new Long(s.getTimestamp().getTime());
		Long oldTime = new Long(lastPlacedTimestamp.getTime());
		lastPlacedTimestamp = s.getTimestamp();
		long difference = newTime.longValue() - oldTime.longValue();
		
		if(difference > MAX_GAP_MILLIS)
			return MAX_HEIGHT;
		float floatHeight = difference;
		floatHeight = (floatHeight / MAX_GAP_MILLIS) * MAX_HEIGHT;		
		Float tempFloat = new Float(floatHeight);
		int tempInt = tempFloat.intValue();
		
		if( tempInt < MIN_HEIGHT ){
			return MIN_HEIGHT;
		}
		//System.out.println("Float: "+floatHeight);
		//System.out.println("Int: "+tempInt.intValue());
		
		return tempInt;
	}
	
	private Color getTypeColor(int type){
		switch(type){
			case Snapshot.ADD_SCRAP: return Color.blue;
			case Snapshot.ADD_PREMADE: return Color.blue;
			case Snapshot.LIFT_SCRAP: return Color.blue;
			case Snapshot.DUPLICATE_SCRAP: return Color.blue;
			
			case Snapshot.ADD_RELATIONSHIP: return new Color((float)0.5, (float)0.5, (float)1.0);
			
			case Snapshot.DRAW: return Color.green;
			case Snapshot.CREATE_BACKGROUND_ELEMENT: return Color.green;
			
			case Snapshot.DELETE_SCRAP: return Color.red;
			case Snapshot.DELETE_RELATIONSHIP: return Color.red;
			case Snapshot.ERASE_BACKGROUND_ELEMENT: return Color.red;
			
			case Snapshot.MODIFY_SCRAP: return Color.pink;
			case Snapshot.MODIFY_RELATIONSHIP: return Color.pink;
			
			case Snapshot.MOVE_SCRAP: return Color.magenta;
			
			case Snapshot.SAVE: return Color.darkGray;
			case Snapshot.LOAD: return Color.darkGray;
			case Snapshot.GO_DOWN: return Color.darkGray;
			case Snapshot.GO_UP: return Color.darkGray;
			case Snapshot.LOAD_STATE: return Color.darkGray;
						
			case Snapshot.DESELECT: return Color.lightGray;
			case Snapshot.SELECT_SCRAP: return Color.lightGray;
						
			case Snapshot.DROP_SCRAP: return Color.orange;					
			case Snapshot.TRANSFORM_SCRAP_TO_RECT: return Color.yellow;
		}		
		return Color.black;
	}
	
	public void setHistory(SnapshotHistory sh){
		history = sh;
	}
	
	private class BarClickListener implements MouseListener {
	    public BarClickListener(){
	    	return;}
	    public void mousePressed(MouseEvent e) {
	    	int temp = findSnapshotIndex(e.getX());
		    if(temp < 0){
		    	return;}
	    	System.out.println("Showing snapshot :"+temp);
		    reader.placeImage(history.getSnapshotAt(temp));
		    
		    if(viewIndicatorX > -1){
		    	reader.repaint(viewIndicatorX - 1, 0, 3, BUFFER + VIEW_INDICATOR_HEIGHT + 1);}
		    viewIndicatorX = e.getX();
		    reader.repaint(e.getX() - 1, 0, 3, BUFFER + VIEW_INDICATOR_HEIGHT + 1);
	    }
	    public void mouseReleased(MouseEvent e) {}
	    public void mouseEntered(MouseEvent e) {}
	    public void mouseExited(MouseEvent e) {}
	    public void mouseClicked(MouseEvent e) {}
	    
	    private int findSnapshotIndex(int x){
	    	int tempLocation = 0;
	    	int tempX = BUFFER;
	    	for(int i = 0; i < history.getSize(); i++){
				Snapshot temp;
	    		temp = history.getSnapshotAt(i);
				
				//Movement lines
				if( temp.getGridCell() != tempLocation){
					tempX += MOVE_LINE_WIDTH;
					tempLocation = temp.getGridCell();
				}			
				
				//Action bars
				else{
					tempX += BAR_WIDTH;
				}
				if(tempX > x){
					return i;
				}
			}
	    	return -1;
	    }
	}
}


