package ics.uci.edu.Calico.Readers;

import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.models.Snapshot;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.awt.FileDialog;
import javax.swing.*;
import edu.umd.cs.piccolo.PLayer;
import java.io.*;

//TODO
// * Allow it to notice and display changes in grid location, may need to change snapshot itself
// * Add heights based on time gap

public class FilmstripReader extends JFrame{
//VBActionListener
   /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private BarCanvas barCanvas;
	private JScrollPane barScrollPane;
	private CalicoCanvas viewerCanvas;
	private SnapshotHistory history;
   	private final static int OVERALL_WIDTH = 1024;
	private final static int OVERALL_HEIGHT = 768;
	private final static int BAR_HEIGHT = 80;
	private final static int SCROLL_PANE_HEIGHT = BAR_HEIGHT + 5;
	private final static int VIEWER_HEIGHT = 500;
	
	private final static int VIEW_WIDTH = 600;
	private final static int VIEW_HEIGHT = 400;
	private final static int VIEW_X_OFFSET = (OVERALL_WIDTH - VIEW_WIDTH) / 2;
	private final static int VIEW_Y_OFFSET = (VIEWER_HEIGHT - VIEW_HEIGHT) / 2;
	
	protected MenuListener menuListener;
	protected int currentCell;
	protected FileDialog saveDialog;
	protected FileDialog loadDialog;
	
   public FilmstripReader(){ //Begin Constructor
       	setTitle("VBDDReader - Alpha 0.01");
       	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);	
       	history = new SnapshotHistory();       	

       	//Menu stuff
        JMenuBar menuBar;
        JMenu menu;
        JMenuItem menuItem;
        menuListener = new MenuListener(this);

        menuBar = new JMenuBar();
        menu = new JMenu("File");
        menu.setMnemonic(KeyEvent.VK_F);
        menuBar.add(menu);

        menuItem = new JMenuItem("Save", KeyEvent.VK_S);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.ALT_MASK));
        menuItem.setActionCommand("Save");
        menuItem.addActionListener(menuListener);
        menu.add(menuItem);
        
        menuItem = new JMenuItem("Load", KeyEvent.VK_L);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.ALT_MASK));
        menuItem.setActionCommand("Load");   
        menuItem.addActionListener(menuListener);
        menu.add(menuItem);
        
        this.setJMenuBar(menuBar);
		saveDialog = new FileDialog(this, "Save", FileDialog.SAVE);
        loadDialog = new FileDialog(this, "Load", FileDialog.LOAD);
       	
        //Canvases
        barCanvas = new BarCanvas(this, history);
       	barCanvas.setPreferredSize(new Dimension(OVERALL_WIDTH, BAR_HEIGHT));
       	
       	viewerCanvas = new CalicoCanvas();
       	viewerCanvas.setPreferredSize(new Dimension(OVERALL_WIDTH, VIEWER_HEIGHT));
        
        //Add it all in
		getContentPane().setLayout(new BorderLayout());
		barScrollPane = new JScrollPane(barCanvas);
		barScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		barScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
		barScrollPane.setPreferredSize(new Dimension(OVERALL_WIDTH, SCROLL_PANE_HEIGHT));
		
        getContentPane().add(barScrollPane, BorderLayout.PAGE_START);
        getContentPane().add(viewerCanvas, BorderLayout.PAGE_END);
        
        pack();
        setVisible(true);
        
        loadHistory();
        
        barCanvas.setHistory(history);
        barCanvas.repaint();
        barScrollPane.repaint();
   }

  public void loadHistory(){
	  history = new SnapshotHistory();
	  String filename;
	  Snapshot temp;
	  boolean moreSnapshots = true;
	  
	  loadDialog.setVisible(true);
	  filename = (loadDialog.getDirectory() + loadDialog.getFile());
	  
	  FileInputStream fis = null;
	  ObjectInputStream in = null;
	  
	  //***Open it
	  try
	  {
		  fis = new FileInputStream(filename);
	      in = new ObjectInputStream(fis);    
	  }	  
	  catch(IOException ex)
	  	{ex.printStackTrace();}
	
	  //***Grab all the snapshots
	  while(moreSnapshots){ 
		  try{	      
			  Object instreamObject = in.readObject();
			  if (instreamObject instanceof Snapshot) {
				  try {
					  temp = (Snapshot)in.readObject();
					  history.addSnapshot(temp);
				  }
				  catch (InvalidClassException ice) {
					  System.err.println(ice.toString());
				  }
				  
			  }
			  else
				  System.err.println("Error in FilmstripReader:loadHistory - improper object read... " + instreamObject.toString());
		      
		      //System.out.println("Found snapshot of type: " + temp.getType());
		  }		  
		  
		  catch (EOFException ex){
			  moreSnapshots = false;
			  //System.out.println("ENDING INPUT!");
			  try{
				  in.close();
			  }
			  catch(IOException ex2)
			  	{ex2.printStackTrace();}
			  }
		  catch(IOException ex)
		  	{moreSnapshots = false;
			ex.printStackTrace();}
		  catch(ClassNotFoundException ex)
		  	{moreSnapshots = false;
		  	ex.printStackTrace();}	  
	  }
	  
	  barCanvas.setHistory(history);
	  barCanvas.setPreferredSize(new Dimension(barCanvas.getWidth(), SCROLL_PANE_HEIGHT));
	  barCanvas.revalidate();
	  barScrollPane.revalidate();
	  barScrollPane.repaint();
	  //Needed to get around not serializing the painters
  }
  
	public void placeImage(Snapshot s){
		viewerCanvas.getCamera().removeAllChildren();
		Image tempImage;		
		PLayer tempLayer = s.getLayer();
		
		if (tempLayer.getChildrenCount() == 0) {
			BufferedImage img = new BufferedImage(VIEW_WIDTH, VIEW_HEIGHT, BufferedImage.TYPE_INT_ARGB);
			tempImage = img;
		}
		else 
			tempImage = tempLayer.toImage();		
		tempImage = tempImage.getScaledInstance(VIEW_WIDTH, VIEW_HEIGHT, Image.SCALE_SMOOTH);
		
		LoggerImage newImage = new LoggerImage(s, tempImage,
				new Rectangle(VIEW_X_OFFSET, VIEW_Y_OFFSET, VIEW_WIDTH, VIEW_HEIGHT));
		
		newImage.setBounds(VIEW_X_OFFSET, VIEW_Y_OFFSET, VIEW_WIDTH, VIEW_HEIGHT);
		viewerCanvas.getCamera().addChild(newImage);
	}
  
   public static void main(String[] args){
	   FilmstripReader frame = new FilmstripReader();
   }
}
