package ics.uci.edu.Calico.UI;

import ics.uci.edu.Calico.UI.components.*;
import ics.uci.edu.Calico.UI.components.palettes.*;
import ics.uci.edu.Calico.models.CalicoLogger;
import ics.uci.edu.Calico.UI.CalicoOptions;
import ics.uci.edu.Calico.networking.Networking;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;

public class Calico extends Networking {
	
	private static final long serialVersionUID = 1L;
	CalicoCanvas canvas;
	CalicoLogger logger;
	public boolean LOGGING = true;
	public static final float VERSION = 2.0013f;
	
	public Calico(String[] args)
	{
		String configFile = "config.cfg";
		if( args.length > 1 )
		{
			if(args[0].equals("-c") || args[0].equals("--config") )
			{
				configFile = args[1];
			}
		}
		
		CalicoOptions.readConfig(configFile);
		setupCalico();
	}
		
	private void setupCalico()
	{
		Dimension fullScreen;
		if(CalicoOptions.calico_fullscreen)
		{
			
			fullScreen = Toolkit.getDefaultToolkit().getScreenSize();
	        setBounds(0,0,fullScreen.width,fullScreen.height); 
	        setUndecorated(true);
		}
		else
		{
			fullScreen = new Dimension(CalicoOptions.calico_screenwidth,CalicoOptions.calico_screenheight);
			setBounds(50,50, fullScreen.width, fullScreen.height);
		}
		
		
		
        setTitle("Calico V" + Calico.getVersion());
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        //Setup Canvas
		canvas = new CalicoCanvas();
		canvas.setPreferredSize(fullScreen);
		
		//Setup Logger
        initializeLogger();
		canvas.addCalicoActionListener(logger);
		

		//Setup Menu
		Menu menu = new Menu();
		

		
		//canvas.addComponent(new Zoom());
		canvas.addComponent(new Undo());
		if(CalicoOptions.modules_active.contains("PENCOLOR"))
		{
			canvas.addComponent(new PenColorChooser());
		}
		if(CalicoOptions.modules_active.contains("PAN"))
		{
			canvas.addComponent(new Panning());
		}
		if(CalicoOptions.modules_active.contains("ZOOM"))
		{
			canvas.addComponent(new ZoomSlider());
		}
		//canvas.addComponent(new Navigator());
		
		if(CalicoOptions.modules_active.contains("PALETTES"))
		{
			canvas.addComponent(new Palettes());
		}
		
		CalicoGrid grid = new CalicoGrid(CalicoOptions.grid_rows,CalicoOptions.grid_cols, menu);
		this.setCalicoGrid(grid);//FOR NETWORKING
		grid.setCalicoInstance(this);
		grid.enableMenu(this);
		canvas.addComponent(grid);
		
		//Setup autosave
		Autosave autosave = new Autosave(grid);
		canvas.addCalicoActionListener(autosave);
		
		// Add the node to the canvas layer so that it
		// will be displayed on the screen.
		canvas.removeInputEventListener(canvas.getPanEventHandler());
		canvas.removeInputEventListener(canvas.getZoomEventHandler());
		//canvas.addInputEventListener(new MouseInputHandler(canvas));
		
		getContentPane().add(canvas);
        pack();
        
        grid.loadGrid();
        
        setVisible(true);
        
        //When the window closes, finalize - which right now is just closing the output stream
    	addWindowListener(new java.awt.event.WindowAdapter() {
        	public void windowClosing(WindowEvent winEvt) {
        		// Perhaps ask user if they want to save any unsaved files first.
       		    logger.closeStream();
        		System.exit(0); 
        		}
        });
	}
	
	public void initializeLogger() {
		//Might later include other functions
		logger = new CalicoLogger();
	}
	
	public CalicoLogger getLogger(){
		return logger;
	}
	
	public static void main(String[] args) {
		new Calico(args);
	}

	public static String getVersion() {
		return new Float(VERSION).toString();
	}

	/*
	public void splitSelect(ObjectHandlerModel model, Image img) {
		Selector selector = new Selector(canvas.getImage(canvas.getBackground()), img, model, getWidth(), getHeight(), canvas);
		canvas.getCamera().addChild(selector);
		selector.setBounds();
		
	}*/
	

}
