package ics.uci.edu.Calico.UI;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.Arc2D;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import ics.uci.edu.Calico.Structures.RelationshipController;
import ics.uci.edu.Calico.Structures.ScrapController;
import ics.uci.edu.Calico.models.ScrapModel;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PPaintContext;
import java.io.Serializable;

public class ScrapPainter extends PNode implements Serializable{
	
	private static final long serialVersionUID = 1L;
	private ScrapModel scrap;
	private Polygon shadow;
	private boolean selected = false;
	private boolean widgetOpen = false;
	private boolean translucent = false;

	private ScrapWidget widg;
	
	public ScrapPainter(ScrapModel s) {
		super();
		scrap = s;
		setWidget();
		//addInputEventListener(new ScrapInputHandler());
	}
	
	public boolean containsModel(ScrapModel s) {
		return scrap.equals(s);
	}
	
	public void translate(int deltaX, int deltaY) {
		ScrapController.translate(scrap, new Double(deltaX).intValue(), new Double(deltaY).intValue());
		//move children
		/*
		for (int i = 0; i < getChildrenCount(); i++) {
			if (getChild(i) instanceof ScrapWidget) {
				ScrapWidget sw = (ScrapWidget)getChild(i);
				removeChild(sw);
				sw.translate(deltaX, deltaX);
				addChild(sw);
			}
			//getChild(i).translate(deltaX, deltaY);
			//Rectangle  r = getChild(i).getBounds().getBounds();
			//getChild(i).setBounds(r.getX() + deltaX, r.getY() + deltaY, r.getWidth(), r.getHeight());
		}*/
		//setWidget();
		
		//move connected relationships
		for (int i = 0; i < getParent().getChildrenCount(); i++) {
			if (getParent().getChild(i) instanceof RelationshipPainter &&
					RelationshipController.connectedTo(((RelationshipPainter)getParent().getChild(i)).getModel(), ScrapController.getId(scrap))) {
				((RelationshipPainter)getParent().getChild(i)).translate(ScrapController.getId(scrap), deltaX, deltaY);
			}
		}
		
		setBounds();
	}
	
	/**
	 * Translates ScrapModel scrap so that it's midpoint lies at Point p.
	 * @param scrap
	 * @param p
	 */
	public void translateTo(Point2D p) {
		Point2D midPoint = ScrapController.getMidPoint(scrap);
		int deltaX = new Double(p.getX() - midPoint.getX()).intValue();
		int deltaY = new Double(p.getY() - midPoint.getY()).intValue();
		translate(deltaX, deltaY);
	}

	// This method is important to override so that the geometry of 
	// the ellipse stays consistent with the bounds geometry.
	public boolean setBounds() {
		Rectangle2D bounds = scrap.encasingPoly.getBounds2D();
		setShadow();
		
		if(super.setBounds(bounds.getX()-2, bounds.getY()-2, 
				bounds.getWidth()+4, bounds.getHeight()+4)) {
			//do nothing;
			return true;
		}
		
		setWidget();
		return false;
	}

	// Non rectangular subclasses need to override this method so
	// that they will be picked correctly and will receive the
	// correct mouse events.
	public boolean intersects(Rectangle2D aBounds) {
		return ScrapController.intersects(scrap, aBounds);
	}
	
	public boolean intersectsWidget(Point2D p) {
		for (int i = 0; i < getChildrenCount(); i++)
			if (getChild(i) instanceof ScrapWidget &&
					((ScrapWidget)getChild(i)).contains(p))
				return true;
		return false;
	}
	
	public boolean intersects(Polygon p) {
		return (ScrapController.intersects(scrap.encasingPoly, p) || containedIn(p));
	}
	
	public boolean containedIn(Polygon p) {
		
		for (int i = 0; i < scrap.encasingPoly.npoints; i++) {
			if (p.contains(scrap.encasingPoly.xpoints[i], scrap.encasingPoly.ypoints[i]))
				return true;
		}
		return false;
	}
	
	public boolean widgetContains(Point2D p) {
		return getScrapWidget().contains(p) || 
			(widgetOpen && getWidgetItem(p) != null);
	}
	
	public void paint(PPaintContext aPaintContext) {
		Graphics2D g2 = aPaintContext.getGraphics();
		g2.setPaint(getPaint());
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
		
		g2.setStroke(new BasicStroke(2f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
		g2.setColor(new Color(0,0,0));
		g2.drawPolygon(shadow);
		
		g2.setColor(scrap.getBackgroundColor());
		
		
		
		
		if (translucent || scrap.transparent) {
			g2.setColor(new Color(scrap.getBackgroundColor().getRed(), scrap.getBackgroundColor().getGreen(),
					scrap.getBackgroundColor().getBlue(), 96));
		}
		else
			g2.setColor(scrap.getBackgroundColor());
		
		
		g2.fill(scrap.encasingPoly);
		g2.setStroke(new BasicStroke(1.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
		if (selected) {
			g2.setColor(scrap.getSelectedBorderColor());
		}
		else {
			g2.setColor(scrap.getBorderColor());
		}
		g2.draw(scrap.encasingPoly);
		g2.setStroke(new BasicStroke(1.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
		for (int i = 0; i < scrap.encasedFigures.size(); i++) {
			Polygon df = scrap.encasedFigures.get(i);
			g2.setColor(scrap.encasedFigureColors.get(i));
			
			g2.drawPolyline(df.xpoints, df.ypoints, df.npoints);
		}
	}
	
	public void select() {
		selected = true;
		repaint();
	}
	
	public void deselect() {
		selected = false;
		repaint();
	}
	
	public int getDrawingPriority() {
		PNode p = getParent();
		for (int i = 0; i < p.getChildrenCount(); i++) {
			if (p.getChild(i).equals(this))
				return i;
		}
		return -1;
	}
	
	public static ArrayList<ScrapPainter> getSelected(PNode parent) {
		ArrayList<ScrapPainter> selected = new ArrayList<ScrapPainter>();
		for (int i = 0; i < parent.getChildrenCount(); i++) {
			if (parent.getChild(i) instanceof ScrapPainter &&
					((ScrapPainter)parent.getChild(i)).selected )
				selected.add((ScrapPainter)parent.getChild(i));
		}
		
		return selected;
	}
	
	public ScrapModel getModel() {
		return scrap;
	}

	public boolean contains(Point2D position) {
		return ScrapController.containsPoint(scrap, position) ||
			widgetContains(position);
	}

	public boolean isSelected() {
		return selected;
	}
	
	public boolean isTransparent() {
		return scrap.transparent;
	}
	
	/**
	 * This applies to dragging only
	 * @param trans
	 */
	public void setTranslucent(boolean trans) {
		translucent = trans;
	}
	
	/**
	 * Applies universely
	 * 
	 * @param trans
	 */
	public void setTransparency(boolean trans) {
		scrap.transparent = trans;
		setWidget();
	}
	
	private void setShadow() {
		shadow = new Polygon();
		for (int i = 0; i < scrap.encasingPoly.npoints; i++) {
			shadow.addPoint(scrap.encasingPoly.xpoints[i] + 1, scrap.encasingPoly.ypoints[i] + 1);
		}
	}
	
	public void setWidget() {
		
		for (int i = 0; i < getChildrenCount(); i++)
			if (getChild(i) instanceof ScrapWidget)
				removeChild(i);
		
		
		int adjustedX, adjustedY, highestPoint = 0, highestIndex = -1;
		
		for (int i = 0; i < scrap.encasingPoly.npoints; i++) {
			adjustedX = scrap.encasingPoly.xpoints[i] - new Double(getBounds().getX()).intValue();
			adjustedY = scrap.encasingPoly.ypoints[i] - new Double(getBounds().getY()).intValue();
			
			if (adjustedX + adjustedY > highestPoint) {
				highestPoint = adjustedX + adjustedY;
				highestIndex = i;
			}
		}
		//get point between bottom most and right most
		int midPointIndex = highestIndex;
		Point2D.Double midPoint = new Point2D.Double(scrap.encasingPoly.xpoints[midPointIndex],
											scrap.encasingPoly.ypoints[midPointIndex]);
		widg = new ScrapWidget(midPoint);
		addChild(widg);
	}
	
	private ScrapWidget getScrapWidget() {
		for (int i = 0; i < getChildrenCount(); i++)
			if (getChild(i) instanceof ScrapWidget)
				return (ScrapWidget)getChild(i);
		return null;
	}
	
	public ScrapWidgetItem getWidgetItem(Point2D p) {
		if (!widgetOpen) {
			return null;
		}
		
		for (int i = 0; i < getChildrenCount(); i++)
			if (getChild(i) instanceof ScrapWidgetItem &&
					((ScrapWidgetItem)getChild(i)).getBounds().contains(p))
				return (ScrapWidgetItem)getChild(i);
		
		return null;
	}
	
	public void openWidgetMenu() {
		for (int i = 0; i < getChildrenCount(); i++)
			if (getChild(i) instanceof ScrapWidgetItem) {
				//System.err.println("ScrapPainter:openWidgetMenu - widget menu already open");
				closeWidgetMenu();
				repaint();
				break;
			}
		int centerX = new Double(getScrapWidget().getBounds().getX() + getScrapWidget().getOffset().getX() + 
				getScrapWidget().getBounds().getWidth()/2).intValue();
		int centerY = new Double(getScrapWidget().getBounds().getY() + getScrapWidget().getOffset().getY() + 
				getScrapWidget().getBounds().getHeight()/2).intValue();
		int offset = 40;
		
		ScrapWidgetItem up = new ScrapWidgetItem(ScrapWidgetItem.DUPLICATE);
		up.centerBoundsOnPoint(centerX, centerY - offset);
		ScrapWidgetItem down = new ScrapWidgetItem(ScrapWidgetItem.DROP);
		down.centerBoundsOnPoint(centerX, centerY + offset);
		ScrapWidgetItem transparent = new ScrapWidgetItem((scrap.transparent)?ScrapWidgetItem.OPAQUE:ScrapWidgetItem.TRANSPARENT);
		transparent.centerBoundsOnPoint(centerX - offset, centerY);
		ScrapWidgetItem right = new ScrapWidgetItem(ScrapWidgetItem.RECTANGLE);
		right.centerBoundsOnPoint(centerX + offset, centerY);
		
		addChild(up);
		addChild(down);
		addChild(transparent);
		addChild(right);	
		
		widgetOpen = true;
	}
	
	public void closeWidgetMenu() {
		for (int i = getChildrenCount() - 1; i >= 0; i--)
			if (getChild(i) instanceof ScrapWidgetItem)
				removeChild(i);
		
		widgetOpen = false;
	}
	
	public boolean widgetOpen() {
		return widgetOpen;
	}
	
	public class ScrapWidget extends PNode {
		
		int radius = 6;
		Arc2D circle;
		
		public ScrapWidget(Point2D midPoint) {
			setSize(midPoint, 1);
			
		}
		
		private void setSize(Point2D midPoint, int zoomFactor) {
			setBounds(midPoint.getX() - radius, midPoint.getY() - radius,
					radius * 2, radius * 2);
			circle = new Arc2D.Double(midPoint.getX() - radius, midPoint.getY() - radius,
					radius * 2, radius * 2, 0, 360, Arc2D.PIE);
		}
		
		public void paint(PPaintContext c) {
			Graphics2D g2 = c.getGraphics();
			g2.setPaint(getPaint());
			g2.setColor(new Color(170,170,130));
			
			g2.fill(circle);
		}
		
		public boolean contains(Point2D p) {
			return circle.contains(p);
		}
	}
	
	public class ScrapWidgetItem extends PText {
		
		public static final String DROP = " Drop ";
		public static final String DUPLICATE = " Duplicate ";
		public static final String TRANSPARENT = " Transparent ";
		public static final String OPAQUE = " Opaque ";
		public static final String RIGHT = " Right ";
		public static final String ELLIPSE = " Ellipse ";
		public static final String RECTANGLE = " Rectangle "; 
		
		public ScrapWidgetItem(String s) {
			super(s);
			this.setConstrainHeightToTextHeight(true);
		}
		
		public void paint(PPaintContext c) {
			
			Graphics2D g2 = c.getGraphics();
			g2.setPaint(getPaint());
			
			int x = new Double(getBounds().getX()).intValue(),
				y = new Double(getBounds().getY()).intValue(),
				width = new Double(getBounds().getWidth()).intValue(),
				height = new Double(getBounds().getHeight()).intValue();
			
			g2.setColor(Color.white);
			g2.fillRect(x, y, width, height);
			g2.setColor(Color.black);
			g2.drawRect(x,y,width,height);
			super.paint(c);
		}
	}
	
	public Polygon getTotalPoly() {
		
		Area scrap = new Area(getModel().encasingPoly);
		Area widget = new Area(widg.circle);
		scrap.add(widget);
		return ScrapController.getPolyFromArea(scrap);
	}

	public boolean contains(Polygon poly) {
		boolean contained = true;
		for (int i = 0; i < poly.npoints; i++) {
			if (!scrap.encasingPoly.contains(new Point2D.Double(poly.xpoints[i], poly.ypoints[i])))
				contained = false;
		}
		return contained;
	}
	
}
