package ics.uci.edu.Calico.UI.components;

import java.awt.Image;
import java.awt.Rectangle;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.swing.JFileChooser;

import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.UI.components.CalicoGrid;
import ics.uci.edu.Calico.UI.components.CalicoGrid.CGCell;
import ics.uci.edu.Calico.models.ObjectHandlerModel;
import ics.uci.edu.Calico.models.actions.CalicoActionEvent;
import ics.uci.edu.Calico.models.actions.CalicoActionListener;
import ics.uci.edu.Calico.models.actions.GridView;
import ics.uci.edu.Calico.networking.SerializedCell;

public class Autosave implements CalicoActionListener {

	CalicoGrid grid;
	int counter;
	ExecutorService threadManager;
	boolean firstWrite = true; //is this the first time we're writing this session?
	
	private final static String FILE_NAME_BASE = "$CalicoAuto";
	private final static String FILE_NAME_EXTENSION = "cg";
	
	private String fileName = FILE_NAME_BASE + "1." + FILE_NAME_EXTENSION;
	
	public Autosave(CalicoGrid g) {
		counter = 0;
		threadManager = Executors.newFixedThreadPool( 1 );
		grid = g;
	}
	
	/**
	 * Autosave saves up to 3 previous sessions. The current session gets saved to $CalicoAuto.cg.
	 * Next time the program loads, the autosave from the previous session gets renamed to $CalicoAuto2.cg
	 * This happens up to $CalicoAuto3.cg. Once we get that far, the original $CalicoAuto3.cg gets deleted.
	 */
	private void setupFiles() {
		String file1 = FILE_NAME_BASE + "1." + FILE_NAME_EXTENSION;
		String file2 = FILE_NAME_BASE + "2." + FILE_NAME_EXTENSION;
		String file3 = FILE_NAME_BASE + "3." + FILE_NAME_EXTENSION;

		if (new File(file2).exists()) {
			File f2 = new File(file2);
			if (new File(file3).exists())
				new File(file3).delete();
			f2.renameTo(new File(file3));
		}
		if (new File(file1).exists()) {
			File f2 = new File(file2);
			new File(file1).renameTo(f2);
		}
	}

	@Override
	public void fireCalicoListener(CalicoActionEvent vbe) {
		if (counter == 5 && grid != null) {
			if (firstWrite) {
				setupFiles();
				firstWrite = false;
			}
			threadManager.execute(new SaveThread());
			
	    	counter = 0;
		}
		else
			counter++;
	}
	
	private class SaveThread implements Runnable {

		@Override
		public void run() {
			//get cells from grid
			ArrayList<SerializedCell> cells = new ArrayList<SerializedCell>();
			int activeCellIndex = -1;
			for (int i = 0; i < grid.getChildrenCount(); i++) {
				if (grid.getChild(i) instanceof CGCell &&
						((CGCell)grid.getChild(i)).model != null &&
						((CGCell)grid.getChild(i)).model.objectIdTicker > 0) {
					cells.add(new SerializedCell((CGCell)grid.getChild(i)));
					if (cells.get(cells.size()-1).position == grid.activeCanvasPosition)
						activeCellIndex = i;
				}
			}
			//The user hasn't left the first cell
			if (cells.size() == 0) {
				cells.add(new SerializedCell(getUpdatedActiveCell(grid.activeCanvasPosition)));
			}
			//the user is looking at the grid
			if (activeCellIndex == -1) {
				
			}
			//update the active cell
			else {			
				
				cells.set(activeCellIndex, new SerializedCell(getUpdatedActiveCell(cells.get(activeCellIndex).position)));
			}
			//save the file
			File file = new File(fileName);
	    	saveGrid(file, cells);
			
			
		}
		
		private CGCell getUpdatedActiveCell(int pos) {
			CGCell tempCell = grid.new CGCell(pos);
			ObjectHandlerModel tempModel = grid.canvas.getModel();
			CalicoCanvas tempCanvas = new CalicoCanvas();
			tempCanvas.loadModel(tempModel);
			tempCanvas.setBounds(grid.canvas.getBounds());
			
			
			
			tempCell.setModel(tempCanvas.getModel());
			Image img = tempCanvas.getImage(tempModel.bgColor);
			
			int height = new Double(grid.getBounds().getHeight() / grid.numRows).intValue();
			int width = new Double(grid.getBounds().getWidth() / grid.numColumns).intValue();
			if (img != null)
				tempCell.setImage(CalicoCanvas.getProportionateScaledImage(img, width, height, Image.SCALE_SMOOTH));
			if (tempModel != null)
				tempCell.setModel(tempModel);
			Rectangle bounds = grid.getCellBounds(pos);
			//tempCell.setImage(tempCanvas.getImage(tempCell.model.bgColor));
			tempCell.setBounds(bounds);
			
			return tempCell;
		}
		
		private void saveGrid(File f, ArrayList<SerializedCell> cells) {
			try {
		        
			    if (f == null)
			    	return;
		        ObjectOutput out = new ObjectOutputStream(new FileOutputStream(f));
		        out.writeObject(cells);
		        out.close();
		    } catch (IOException e) {
		    	System.err.println(e.getMessage());
		    }
		}
		
		
	}
}
