package ics.uci.edu.Calico.UI.components;

import ics.uci.edu.Calico.UI.Calico;
import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.UI.MouseInputHandler;
import ics.uci.edu.Calico.models.ObjectHandlerModel;
import ics.uci.edu.Calico.models.actions.BGColorChanged;
import ics.uci.edu.Calico.models.actions.CalicoAction;
import ics.uci.edu.Calico.models.actions.CalicoActionEvent;
import ics.uci.edu.Calico.models.actions.CalicoActionListener;
import ics.uci.edu.Calico.models.actions.ChangedCell;
import ics.uci.edu.Calico.models.actions.GridCellChanged;
import ics.uci.edu.Calico.models.actions.GridView;
import ics.uci.edu.Calico.networking.Networking;
import ics.uci.edu.Calico.networking.SerializedCell;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;
import java.awt.geom.RoundRectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;

import edu.umd.cs.piccolo.nodes.PImage;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolo.util.PPaintContext;
import edu.umd.cs.piccolox.nodes.PLine;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;

/**
 * This class is a grid of CalcioGridCells, which in themselves have an image and an ObjectHandlerModel object.
 * There is a set number of cells that this can have, declared by MAX_ROWS and MAX_COLUMNS, but only a fraction
 * of these are intended to be visible initially. The visual matrix will resize itself if more columns are requested
 * but it will return bad values when the absolute walls are hit (getAdjacentCellPosition is where the resizing
 * of the visual grid happens).
 * 
 * Each grid cell has a position number (which can be obtained through get methods). The numbering of the positioning
 * is based off the theoretical max number of cells, so even if there is only a 4x4 matrix visible, the numbering
 * is based off the maximum grid size. This is done to preserve an individual cell's position number as the visual
 * grid increases. Meep.
 * 
 * @author nmangano
 *
 */
public class CalicoGrid extends CalicoComponent implements CalicoActionListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	public static int NORTH = 0;
	public static int SOUTH = 1;
	public static int EAST = 2;
	public static int WEST = 3;
	
	public static final int MAX_ROWS = 64;
	public static final int MAX_COLUMNS = 64;
	
	//these four variables constitute the "Visual Grid". The visual grid has its upper-left corner specified by
	//row0,col0
	int row0, col0,
		numRows, numColumns;
	int currentCell;
	CalicoCanvas canvas;
	Networking frame;
	Calico calicoInstance;
	int activeCanvasPosition;
	JPopupMenu popup;
	int mousedOverCell = -1;
	boolean menuFocus = false;
	MouseInputHandler inputHandler;
	
	Menu menu;
	
	/**
	 * @param row
	 * @param col
	 */
	public CalicoGrid(int row, int col, Menu m) {
		setGridSize(row, col);
		row0 = MAX_ROWS/2 - numRows/2;
		col0 = MAX_COLUMNS/2 - numColumns/2;
		currentCell = getStartingCell();
		activeCanvasPosition = -1;
		menu = m;
		this.addInputEventListener(new GridMouseListener());
	}
	
	public void setCalicoInstance(Calico calico){
		calicoInstance = calico;
		setupRightClickPopup();
	}

	public static int getStartingCell() {
		return (MAX_ROWS * MAX_COLUMNS)/2 - MAX_COLUMNS/2;
	}
	
	public void paint(PPaintContext context) {
		Graphics2D g2 = context.getGraphics();
		//g2.setPaint(Color.white);
		g2.setColor(new Color(230, 230, 230));
	
		Rectangle r = this.getBounds().getBounds();
		g2.fill(r);
	}
	
	public void clear() {
		removeAllChildren();
		readjustGrid();
	}
	
	/**
	 * Overrides the super class so the cells are repositioned and resized according to the window.
	 */
	public boolean setBounds(Rectangle rect) {	
		
		boolean ret = false;
		if (super.setBounds(rect))
			ret = true;
		readjustGrid();
		return ret;
	}
	
	/**
	 * Warning, this clears the matrix! This method creates the grid of lines on the matrix (which is a set
	 * of PLines).
	 * @param row
	 * @param col
	 */
	private void setGridSize(int row, int col) {
		removeAllChildren();
		
		numRows = row;
		numColumns = col;
		
		loadEmptyCells();
		//setNumRows(row);
		//setNumCols(col);
	}
	
	/**
	 * Called by setGridSize()
	 * @param r
	 */
	private void setNumRows(int r) {
		PLine line;
		//double cellHeight = new Double(getBounds().getHeight() / r).intValue();
		double cellHeight = getCellHeight();
		
		for (int i = 1; i < r; i++) {
			line = new PLine();
			line.addPoint(0, 0, cellHeight * i);
			line.addPoint(1, getBounds().getWidth(), cellHeight * i);
			line.setStroke(new BasicStroke(1.0f));
			addChild(line);
		}
	}
	
	/**
	 * Called by setGridSize()
	 * @param c
	 */
	private void setNumCols(int c) {
		PLine line;
		//double cellWidth = new Double(getBounds().getWidth() / c).intValue();
		double cellWidth = getCellWidth();
		
		for (int i = 1; i < c; i++) {
			line = new PLine();
			line.addPoint(0, cellWidth * i, 0);
			line.addPoint(1, cellWidth * i, getBounds().getHeight());
			line.setStroke(new BasicStroke(1.0f));
			addChild(line);
		}
	}
	
	
	
	//TODO CHECK ME
	public void updateCells(ArrayList<SerializedCell> cells){
	
	        for (SerializedCell cell : cells) {
	        	
	        	if(cell.position != activeCanvasPosition){ //&& !cell.lastAction.equals(canvas.getLastAction()) ){
	        		
		        	CalicoCanvas tempCanvas = new CalicoCanvas();
		        	tempCanvas.setBounds(canvas.getBounds());//credit: nick
		        	tempCanvas.loadModel(cell.model);
		        	
		        	System.out.println("updating cell: " + cell.position + " It has " + canvas.getListeners().size() + " listeners and "
		        			+ cell.inhabitants + " inhabitants");
		        	
		        	this.setCell(cell.position, tempCanvas.getLayer().toImage(new Double(this.getBounds().getWidth()).intValue(), 
		        			new Double(this.getBounds().getHeight()).intValue(), tempCanvas.getBackground()), cell.model, cell.inhabitants);
		        	}
	        }
	        //this.repaint();
	}
	/*public void updateCell(int position, CalicoAction action){
		System.out.println("updating cell: " + position + ". listeners: " + canvas.getListeners().size());
		CalicoActionListener actionListener = null;
		  
		for (CalicoActionListener listener : canvas.getListeners())
			  if (listener instanceof Networking)
				  actionListener = listener;
		  
		  canvas.removeCalicoActionListener(actionListener);
		  canvas.applyAction(action);
		  canvas.repaint();
		  canvas.addCalicoActionListener(actionListener);
		  this.repaint();
	}*/
	
	
	//TODO
	public void updateCell(int position, CalicoAction action){
		CGCell tempCell = this.getCellByPosition(position);
		ObjectHandlerModel tempModel = tempCell.getModel();
		CalicoCanvas tempCanvas = new CalicoCanvas();
		tempCanvas.loadModel(tempModel);
		tempCanvas.applyAction(action);
		tempCell.setModel(tempCanvas.getModel());
		
	}
	
	public void updateCell(SerializedCell sC){
		CGCell tempCell = this.getCellByPosition(sC.position);
		this.setCell(tempCell.position,canvas.getImage(tempCell.model.bgColor), sC.model, tempCell.inhabitants);
		
	}
	
	/**
	 * This is the principle method to populate the matrix. If a cell already exists, it will be over-written.
	 * @param pos
	 * @param img
	 * @param m
	 */
	//TODO FIX THIS FOR MORE PIE

	public void setCell(int pos, Image img, ObjectHandlerModel m, int inhabitants) {

		if (MAX_ROWS*MAX_COLUMNS < pos) {
			System.err.println("CalicoGrid:addImage() - pos of " + pos + " is bigger than the available cells " + MAX_ROWS*MAX_COLUMNS);
			return;
		}
		//remove the existing cell
		CGCell temp = getCellByPosition(pos);
		
		
		if (temp != null){//blah blah
			removeChild(temp);
		}
		
		CGCell image = new CGCell(pos);
		image.setInhabitants(inhabitants);
		
		int height = new Double(getBounds().getHeight() / numRows).intValue();
		int width = new Double(getBounds().getWidth() / numColumns).intValue();
		if (img != null)
			image.setImage(CalicoCanvas.getProportionateScaledImage(img, width, height, Image.SCALE_SMOOTH));
		if (m != null)
			image.setModel(m);
		Rectangle bounds = getCellBounds(pos);

		
		//add it to the canvas
		addChild(image);
		image.setBounds(bounds);
		image.moveToBack();
		//System.out.println("set cell");
		image.repaint();
		
	}
	
	/**
	 * This is a private function that readjusts the cell sizes and redraws the lines. The function is
	 * called when the size of the matrix is changed, or the window is resized.
	 *
	 */
	private void readjustGrid() {
		//grab all the existing cells
		ArrayList<CGCell> cells = new ArrayList<CGCell>();		
		for (int i = 0; i < getChildrenCount(); i++) {
			if (getChild(i) instanceof CGCell) {
				cells.add((CGCell)getChild(i));
				cells.get(i).setBounds(getCellBounds(cells.get(i).position));
			}
		}
		//reset all the lines on the grid (keep in mind this wipes everything!)
		setGridSize(numRows, numColumns);
		
		//put all the resized cells back on the layer (draw them last)
		for (CGCell c : cells) {
			this.setCell(c.position, c.getImage(), c.getModel(), c.inhabitants);
		}
		
		repaint();		
	}
	
	/**
	 * Removes a cell based on its position
	 * @param pos
	 */
	public void removeImage(int pos) {
		if (numRows*numColumns < pos) {
			System.err.println("VBGrid:addImage() - pos of " + pos + " is bigger than the available cells " + numRows*numColumns);
			return;
		}
		
		//figure out where the point would be, and use the other algorithm
		Rectangle bounds = getCellBounds(pos);
		Point2D p = new Point2D.Double(bounds.getX()+1, bounds.getY()+1);
		removeImage(p);
	}
	
	
	/**
	 * Removes a cell based on its visual position.
	 * @param p
	 */
	public void removeImage(Point2D p) {
		for (int i = 0; i < getChildrenCount(); i++) {
			if (getChild(i) instanceof CGCell &&
					((CGCell)getChild(i)).getBounds().contains(p))
				removeChild(i);
			break;
		}
	}
	
	private void loadEmptyCells() {
		int pos;
		for (int x = 1, y = 1; getPositionByPoint(x,y) != -1; x += getCellWidth()) {
			for (y = 1; getPositionByPoint(x,y) != -1; y += getCellHeight()) {
				pos = getPositionByPoint(x,y);
				if (getCellByPosition(pos) == null) {
					//System.out.print(counter + " ");
					//BufferedImage img = new BufferedImage(new Double(getBounds().width).intValue(), new Double(getBounds().height).intValue(), BufferedImage.TYPE_INT_RGB);
					//Graphics2D g = (Graphics2D)img.getGraphics();
					//g.setColor(Color.RED);
					//g.fillRect(0,0, new Double(getBounds().width).intValue(), new Double(getBounds().height).intValue());
					//VBGridMenu.saveImage(img, "EmptyCell.jpg");
					setCell(pos, null, null, 0);
				}
			}
			y = 1;
		}
		
	}
	
	/**
	 * Please use getCellBounds instead.
	 * @param pos
	 * @return
	 */
	@Deprecated
	public Rectangle getPositionBounds(int pos) {
		return getCellBounds(pos);
	}
	
	/**
	 * Returns the bounds of a cell based on its position. If a cell is not in the visual grid, then
	 * null is returned.
	 * @param pos
	 * @return
	 */
	public Rectangle getCellBounds(int pos) {
		int x, y;
		
		Point2D coord = getCoordinate(pos);
		int row = new Double(coord.getX()).intValue();
		int col = new Double(coord.getY()).intValue();
		
		int cellHeight = getCellHeight();
		int cellWidth = getCellWidth();
		
		if (col < col0 || col > col0 + numColumns ||
			row < row0 || row > row0 + numRows )
			return null;
		
		x = cellWidth * (col - col0);
		y = cellHeight * (row - row0);
		
		return new Rectangle(x, y, cellWidth, cellHeight);
		
	}
	
	public int getPositionByPoint(int x, int y) {
		return getPositionByPoint(new Point2D.Double(x,y));
	}
	
	/**
	 * Returns a position based on its visual point.
	 * @param p
	 * @return
	 */
	public int getPositionByPoint(Point2D p) {
		
		int pos;
		
		for (int r = row0; r < row0 + numRows; r++) {
			for (int c = col0; c < col0 + numColumns; c++) {
				pos = (MAX_COLUMNS*(r)) + c;
				if (getCellBounds(pos).contains(p))
					return pos;
			}
		}
		
		return -1;
	}
	
	/**
	 * The principle method for getting cells. Returns a VBGridCell object based on its position.
	 * @param pos
	 * @return
	 */
	public CGCell getCellByPosition(int pos) {
		Rectangle bounds = getCellBounds(pos);
		if (bounds == null)
			return null;
		
		Point2D p = new Point2D.Double(bounds.getX() + 1, bounds.getY() + 1); 
		return getCellByPoint(p);
	}

	/**
	 *
	 * @return Returns a cell width that maintains a consistent ratio with the cell height.
	 */
	public int getCellWidth() {
		double scalingRatio = 1d / Math.max(numRows, numColumns);
		
		return new Double(getBounds().getWidth() * scalingRatio).intValue();
	}
	
	/**
	 * 
	 * @return Returns a cell height that maintains a consistent ratio with the cell width.
	 */
	public int getCellHeight() {
		double scalingRatio = 1d / Math.max(numRows, numColumns);
		
		return new Double(getBounds().getHeight() * scalingRatio).intValue();
	}
	
	/**
	 * Same as getCellByPosition, except it just returns the contained model.
	 * @param pos
	 * @return
	 */
	public ObjectHandlerModel getModelByPosition(int pos) {
		CGCell c = getCellByPosition(pos);
		if (c == null)
			return null;
		else
			return c.getModel();
	}
	
	
	
	
	
	/**
	 * 
	 * @param p
	 * @return The CalicoGrid cell based on the visual point.
	 */
	public CGCell getCellByPoint(Point2D p) {
		for (int i = 0; i < getChildrenCount(); i++) {
			if (getChild(i) instanceof CGCell &&
					((CGCell)getChild(i)).getBounds().contains(p))
				return (CGCell)getChild(i);
		}
		return null;
	}
	
	//TODO
	public void addInhabitantByPoint(Point2D p){
		this.getCellByPoint(p).addInhabitant();
	}
	
	/**
	 * Returns the position number based on its position in the matrix.
	 * @param row
	 * @param col
	 * @return
	 */
	public int getPositionByRowCol(int row, int col) {
			return row*MAX_COLUMNS + col;
	}
	
	/**
	 * 
	 * @param pos The numerical position of the cell.
	 * @return Returns the row, column as a point (where X is row, and Y is the col).
	 */
	public Point2D getCoordinate(int pos) {
		
		int row = pos/MAX_COLUMNS;
		int col = pos - row*MAX_COLUMNS;
		
		return new Point2D.Double(row, col);
	}
	
	/**
	 * Returns the position of the adjacent cell in a specified direction. If a cell in the specific direction
	 * isn't visible, then the visible matrix is expanded. If we're at the very edge, then -1 is returned.
	 * 
	 * @param cell
	 * @param direction
	 * @return
	 */
	public int getAdjacentCellPosition(int cell, int direction) {
		//TODO: finish
		int row = cell/MAX_COLUMNS;
		int col = cell - row*MAX_COLUMNS;
		
		//figure out if it's out of bounds
		boolean outOfBounds = false;
		if (direction == NORTH) {
			if (row == row0)
				outOfBounds = true;
			row--;
		}
		else if (direction == SOUTH) {
			if (row == row0 + numRows - 1)
				outOfBounds = true;
			row++;
		}
		else if (direction == EAST) {
			if (col == col0 + numColumns - 1)
				outOfBounds = true;
			col++;
		}
		else if (direction == WEST) {
			if (col == col0)
				outOfBounds = true;
			col--;
		}
		//if it's out of bounds of the visual grid, readjust the visible grid.
		if (outOfBounds)
			if (direction == NORTH) {
				if (row0 == 0)
					return -1;
				else {
					//expand grid
					row0--;
					numRows++;
					readjustGrid();
				}
			}
			else if (direction == SOUTH) {
				if (row0 + numRows >= MAX_ROWS)
					return -1;
				else {
					//expand grid
					numRows++;
					readjustGrid();
				}
			}
			else if (direction == EAST) {
				if (col0 + numColumns >= MAX_COLUMNS)
					return -1;
				else {
					//expand grid
					numColumns++;
					readjustGrid();
				}
			}
			else if (direction == WEST) {
				if (col0 == 0)
					return -1;
				else {
					//expand grid
					col0--;
					numColumns++;
					readjustGrid();
				}
			}
			
		return getPositionByRowCol(row, col);
	}
	
	/**
	 * Copies a cell in the direction specified. If a cell exists in that direction, then that cell is moved one
	 * over in the same direction (this happens recursively).
	 * @param cell
	 * @param direction
	 * @return The position of the cell copied.
	 */
	public int copyCell(int cell, int direction) {
		CGCell origCell = getCellByPosition(cell);
		int dest = getAdjacentCellPosition(cell, direction);
		if (getCellByPosition(dest) != null &&
				!getCellByPosition(dest).isEmpty())
			copyCell(dest, direction);
		setCell(dest, origCell.getImage(), new ObjectHandlerModel(origCell.model), origCell.inhabitants);
		
		return dest;
	}
	
	public void deactiveCell(){
		activeCanvasPosition = -1;
		System.out.println("Active Cell: " + activeCanvasPosition);
	}
	
	//TODO when grid component is pressed, request for a grid update!
	private class GridMouseListener extends PBasicInputEventHandler {
		
		//private static final int NO_ACTION = -1;
		private static final int TRANSLATE = 1;
		
		CGCell floater;
		PInputEvent mDown;
		private int initX, initY;
		
		
		int action = -1;
		
		public void mousePressed(PInputEvent e) {
			
			mousedOverCell = getCellByPoint(e.getCanvasPosition()).position;
			
			initX = new Double(e.getCanvasPosition().getX()).intValue();
			initY = new Double(e.getCanvasPosition().getY()).intValue();
			
			mDown = e;
			
			
		}
		
		
		
		public void mouseReleased(PInputEvent e) {
			
			boolean cache = menuFocus;
			if (action == TRANSLATE) {
				double midX = floater.getFullBoundsReference().getX() + floater.getFullBoundsReference().getWidth()/2;
				double midY = floater.getFullBoundsReference().getY() + floater.getFullBoundsReference().getHeight()/2;
				//double midX = floater.getX() + floater.getWidth()/2;
				//double midY = floater.getY() + floater.getHeight()/2;
				
				int pos = getPositionByPoint(new Point2D.Double(midX, midY));
				int initPos = getPositionByPoint(new Point2D.Double(initX, initY));
				if (pos != -1 && pos != initPos) {
					//setCell(floater.position, null, null);
					removeChild(floater);
					
					//setCell(floater.position, null, null);
					setCell(floater.position, floater.getImage(), floater.getModel(), floater.inhabitants);
					canvas.applyAction(new GridCellChanged(pos, floater.position));
					
					//removeChild(floater);
					//floater.setBounds(getCellBounds(floater.position).getBounds());
				}
				else {
					floater.setBounds(getCellBounds(floater.position).getBounds());
				}
				repaint();
			}
			else if (e.isRightMouseButton() && 
					getCellByPoint(e.getCanvasPosition()) != null) {
				
				if (getCellByPoint(e.getCanvasPosition()).model != null) {
					popup.show(canvas, new Double(e.getPosition().getX()).intValue(), 
											new Double(e.getPosition().getY()).intValue());
					menuFocus = true;
				}
				
			}
			else if (e.isLeftMouseButton() && 
					(CalicoGrid.this.getCellByPoint(new Point2D.Double(initX,initY)).position == getCellByPoint(e.getCanvasPosition()).position) && !menuFocus) {
				//
				
				//addInhabitantByPoint(e.getCanvasPosition());
				CGCell cell = getCellByPoint(e.getCanvasPosition());
				//TODO fix the inhabitant shit
				//cell.addInhabitant();
				//System.out.println("inhabitants of cell : " + +  cell.getInhabitants());
				
				activeCanvasPosition = cell.position;
				//System.out.println("Active Cell: " + activeCanvasPosition);
				
				int position = getPositionByPoint(e.getCanvasPosition());
				if (cell != null) {
					CalicoCanvas tempCanvas = loadDrawingCanvas(position, cell.getModel());
					calicoInstance.activateClientCanvas(position);
					calicoInstance.setCurrentCalicoCanvas(tempCanvas);
					System.out.println("A");
				}
				else if (position > -1) {
					ObjectHandlerModel emptyModel = new ObjectHandlerModel();
					CalicoCanvas tempCanvas = loadDrawingCanvas(position, emptyModel);
					calicoInstance.activateClientCanvas(position);
					calicoInstance.setCurrentCalicoCanvas(tempCanvas);
					System.out.println("B");
					
				}	
				
			}
			action = -1;
			if (cache == true)
				menuFocus = false;
		}
		
		public void mouseDragged(PInputEvent e) {
			int deltaX = initX - new Double(e.getCanvasPosition().getX()).intValue();
			int deltaY = initY - new Double(e.getCanvasPosition().getY()).intValue();
			
			if (menuFocus) {
				
			}
			else if (action == TRANSLATE) {
				double dX = e.getCanvasDelta().getWidth();
				double dY = e.getCanvasDelta().getHeight();
				floater.translate(dX, dY);
			} else if (e.isRightMouseButton() && (deltaX > 5 || deltaY > 5) &&
					getCellByPoint(new Point2D.Double(initX, initY)) != null) {
				action = TRANSLATE;
				floater = getCellByPoint(new Point2D.Double(initX, initY));
				floater.moveToFront();
				floater.setX(e.getCanvasPosition().getX() - floater.getBounds().getWidth()/2);
				floater.setY(e.getCanvasPosition().getY() - floater.getBounds().getHeight()/2);
				
			}
			
			
		}

	}
	
	/**
	 * The principle unit within the matrix.
	 * @author nmangano
	 *
	 */
	public class CGCell extends PImage implements Serializable {
		
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		ObjectHandlerModel model;
		public int position;
		private Area border, shadow;
		private int inhabitants;
		
		public CGCell(int pos) {
			position = pos;
			inhabitants = 0;
		}
		public boolean isEmpty() {
			return model == null || model.objectIdTicker > -1;
		}
		
		public void setModel(ObjectHandlerModel m) {
			model = m;
		}
		
		public ObjectHandlerModel getModel() {
			if (model == null)
				return new ObjectHandlerModel();
			else
				return model;
		}
		
		public void addInhabitant(){
			System.out.println("add inhabitant");
			inhabitants++;
		}
		
		public void removeInhabitant(){
			inhabitants--;
		}
		
		public int getInhabitants(){
			return inhabitants;
		}
		
		public void setInhabitants(int inhab){
			inhabitants = inhab;
		}
	
		
		public void setImage(Image img) {
			Rectangle rect = new Rectangle(0, 0, img.getWidth(null), img.getHeight(null));
			RoundRectangle2D rRect = new RoundRectangle2D.Double(2, 2, img.getWidth(null) - 4, img.getHeight(null) - 4, 8, 8);
			Area border = new Area(rect);
			border.subtract(new Area(rRect));
			
			BufferedImage bimg = new BufferedImage(img.getWidth(null), img.getHeight(null), BufferedImage.TYPE_INT_ARGB);
			Graphics2D g = (Graphics2D)bimg.createGraphics();
			g.setComposite(AlphaComposite.Src);
			g.drawImage(img, 0, 0, null);
			g.setColor(Color.green);
			g.fill(border);
			g.setColor(new Color(100,100,100));
			g.draw(rRect);
			g.setColor(Color.black);
			g.drawRoundRect(new Double(getBounds().x).intValue() + 2, new Double(getBounds().y).intValue() + 2, 
					new Double(getBounds().width).intValue() - 4, new Double(getBounds().height).intValue() - 4, 10, 10);
			g.dispose();
			int bimgHeight = bimg.getHeight(), bimgWidth = bimg.getWidth();
			for(int i = 0; i < bimgHeight; i++) {
				for(int j = 0; j < bimgWidth; j++) {
					if(bimg.getRGB(j, i) == Color.green.getRGB()) {  
						bimg.setRGB(j, i, 0x8F1C1C);  
					}
				}
			}
			
			
			super.setImage(bimg);
			
		}
		
		public boolean setBounds(Rectangle r) {
			super.setBounds(r);
			Rectangle rect = new Rectangle(r.x, r.y, r.width, r.height);
			RoundRectangle2D rRect = new RoundRectangle2D.Double(r.x + 2, r.y + 2, r.width - 4, r.height - 4, 10, 10);
			border = new Area(rect);
			border.subtract(new Area(rRect));
			
			shadow = new Area(rRect);
			RoundRectangle2D rRect2 = new RoundRectangle2D.Double(r.x + 1, r.y + 1, r.width - 4, r.height - 4, 10, 10);
			shadow.subtract(new Area(rRect2));
			
			return true;
		}
		
		//TODO fix inhabitant stuff
		public void paint(PPaintContext c) {
			
			Image image = getImage();
			Ellipse2D.Double person;
			
			Graphics2D g2 = c.getGraphics();
			//Graphics2D inhabitantGraphics = c.getGraphics();
			
			g2.setStroke(new BasicStroke(1.0f));
			g2.setColor(Color.black);
			//g2.drawRoundRect(new Double(getBounds().x).intValue() + 3, new Double(getBounds().y).intValue() + 3, 
			//		new Double(getBounds().width).intValue() - 4, new Double(getBounds().height).intValue() - 4, 10, 10);
			if (getImage() != null) {
				double iw = image.getWidth(null);
				double ih = image.getHeight(null);
				PBounds b = getBoundsReference();

				if (b.x != 0 || b.y != 0 || b.width != iw || b.height != ih) {
					g2.translate(b.x, b.y);
					g2.scale(b.width / iw, b.height / ih);
					g2.drawImage(image, 0, 0, null);
					g2.scale(iw / b.width, ih / b.height);
					g2.translate(-b.x, -b.y);
				} 
			}
			else {
				g2.setColor(Color.white);
				g2.fillRoundRect(new Double(getBoundsReference().x).intValue() + 2, new Double(getBoundsReference().y).intValue() + 2, 
						new Double(getBoundsReference().width).intValue() - 4, new Double(getBoundsReference().height).intValue() - 4,
						10, 10);
				g2.setColor(new Color(100,100,100));
				
				//g2.drawRoundRect(new Double(getBounds().x).intValue() + 2, new Double(getBounds().y).intValue() + 2, 
				//	new Double(getBounds().width).intValue() - 4, new Double(getBounds().height).intValue() - 4, 10, 10);
			}
			//System.out.println(inhabitants);
			int horizontalAdjustment= 5;
			int xOffset = 5;
			
			
			
			for (int i = 0; i < inhabitants; i++){
				person = new Ellipse2D.Double(getBounds().x+i*horizontalAdjustment + xOffset,getBounds().y+5,3,5);
				g2.setColor(Color.blue);
				g2.fill(person);
			}
			//g2.drawRect(300,300,30,30);
			//g2.fill(border);
			
			//g2.draw(shadow);
			
		}
		
		
	}

	
	public void setupMenu(){
			menu.addConnect(frame);
			menu.addDisconnect(frame);
			menu.addRefreshOption(this);
			//menu.addFileBreak();
			//menu.addSaveOption(canvas, canvas);
			//menu.addLoadOption(canvas);
			menu.addSaveGridOption(this);
			menu.addSaveAllAsImageOption(this);
			menu.addLoadGridOption(canvas);
			
			//menu.addLoggingOption(canvas);
			
			//menu.addFileBreak();
			menu.addExitOption(calicoInstance.getLogger());
			//menu.addToolBreak();
			//menu.toggleLogging(calicoInstance);
			//menu.addStartServer(frame);
			
			//menu.addManagePalettes();
			//menu.addAddPalettes();
			menu.addLoadPalette();
			menu.addSavePalette();
		
	}
	
	
	/**
	 * Sets up the CalicoGrid
	 */
	public void loadGrid() {
		//setupMenu();
		//canvas.removeInputEventListener(inputHandler);
		canvas.clearInvalidNodes();		
		//inputHandler = null;
		canvas.fireListeners(new GridView(currentCell));
		canvas.getCamera().addChild(this);
		updateBounds(canvas.getWidth(), canvas.getHeight());
		saveModelToGrid();
		frame.setJMenuBar(menu);
		frame.pack();
		canvas.repaint();
	}
	
	
	
	public ArrayList<SerializedCell> getCurrentCells(){
		ArrayList<SerializedCell> cells = new ArrayList<SerializedCell>();
		
		
		for (int i = 0; i < this.getChildrenCount(); i++) {
			if (this.getChild(i) instanceof CGCell) {
				cells.add(new SerializedCell((CGCell)this.getChild(i)));
			}
		}
		return cells;
	}
	
	
	public CalicoCanvas loadDrawingCanvas(int pos, ObjectHandlerModel model) {
		canvas.getCamera().removeChild(this);
		canvas.loadModel(model);
		canvas.fireListeners(new ChangedCell(pos, currentCell));
		//reset zoom
		double inverseScale = 1 / canvas.getLayer().getCamera(0).getViewScale();
		canvas.getLayer().getCamera(0).scaleViewAboutPoint(inverseScale * 100.0 / 100.0, canvas.getBounds().getBounds().x/2, canvas.getBounds().getBounds().y/2);
		//reset pan
		Point2D trans = new Point2D.Double(canvas.getLayer().getCamera(0).getViewTransform().getTranslateX(),
				canvas.getLayer().getCamera(0).getViewTransform().getTranslateY());
		canvas.getLayer().getCamera(0).translateView(trans.getX() * -1, trans.getY() * -1);
		
		//reset slider
		for (Object node : canvas.getCamera().getAllNodes())
			if (node instanceof ZoomSlider)
					((ZoomSlider)node).setPercent(100);
		
		currentCell = pos;
		
		
		//canvas.addInputEventListener(inputHandler);
		frame.setJMenuBar(null);
		frame.pack();
		canvas.repaint();
		return canvas;
	}
	
	public void shiftCanvas(int direction, int action) {
		saveModelToGrid();
		int dest = -1;
		
		if (action == GridNavThumb.MOVE) {
			dest = getAdjacentCellPosition(currentCell, direction);
			ObjectHandlerModel model = getModelByPosition(dest);
			if (model == null)
				canvas.loadModel(new ObjectHandlerModel());
			else
				canvas.loadModel(model);
		}
		else if (action == GridNavThumb.COPY) {
			
			dest = copyCell(currentCell, direction);
			
			//we want to disassociate this model from the saved version
			canvas.loadModel(canvas.getModel());
			JOptionPane.showMessageDialog(null, "Your work was successfully copied.");
		}
		canvas.fireListeners(new ChangedCell(dest, currentCell));
		currentCell = dest;
	}

	public void setCurrentGridPoint(int p) {
		currentCell = p;
	}
	
	public void saveModelToGrid() {
		setCell(currentCell, canvas.getImage(canvas.getBackground()), canvas.getModel(), 0);//TODO: fix ME!!
	}
	
	public void enableMenu(Networking f) {
		frame = f;
	}
	
	private void setupRightClickPopup() {
		popup = new JPopupMenu();
		JMenuItem menuItem = new JMenuItem("Set to White");
		menuItem.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					canvas.applyAction(new BGColorChanged(Color.white, mousedOverCell));
					menuFocus = false;
				}
			}
		);
		popup.add(menuItem);
		
		menuItem = new JMenuItem("Set to Blue");
		menuItem.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					canvas.applyAction(new BGColorChanged(new Color(198,226,255), mousedOverCell));
					menuFocus = false;
				}
			}
		);
		popup.add(menuItem);
		
		menuItem = new JMenuItem("Set to Brown");
		menuItem.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					canvas.applyAction(new BGColorChanged(new Color(215,215,131), mousedOverCell));
					menuFocus = false;				
				}
			}
		);
		popup.add(menuItem);
		
		menuItem = new JMenuItem("Set to Lavendar");
		menuItem.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					canvas.applyAction(new BGColorChanged(new Color(230,230,250), mousedOverCell));
					menuFocus = false;				
				}
			}
		);
		popup.add(menuItem);
		
		popup.addSeparator();
		menuItem = new JMenuItem("Save Image");
		menuItem.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					
					try {
				        // Serialize to a file
						String filename = File.separator+".jpg";
						System.out.println(filename);
					    JFileChooser fc = new JFileChooser(new File(filename));
					  
					    fc.setSelectedFile(new File(Menu.getTime() + ".png"));
					    fc.setCurrentDirectory(null);
					    // Show open dialog; this method does not return until the dialog is closed
					    fc.showSaveDialog(canvas);
					    File selFile = fc.getSelectedFile();
					    if (selFile == null)
					    	return;
					    
					    CGCell cell = getCellByPosition(mousedOverCell);
					    CalicoCanvas c = new CalicoCanvas();
						c.setBounds(canvas.getBounds());
						c.loadModel(cell.model);
						
						
					    ImageIO.write((RenderedImage) c.getBufferedImage(cell.model.bgColor), "png", selFile);
				    } catch (IOException ioe) {
				    	System.err.println(ioe.getMessage());
				    }

					menuFocus = false;
				}
			}
		);
		popup.add(menuItem);
		
	
	}
	
	private void refreshCell(CGCell cell) {
		CalicoCanvas c = new CalicoCanvas();
		c.setBounds(canvas.getBounds());
		c.loadModel(cell.model);
		setCell(cell.position, c.getImage(cell.model.bgColor), cell.model, cell.inhabitants);
	}

	@Override
	public void registerCanvas(CalicoCanvas c) {
		canvas = c;
		//We only want this present situationally
		canvas.getCamera().removeChild(this);
		canvas.addComponent(new GridNavPointer());
		canvas.addComponent(new GridNavThumb(GridNavThumb.NORTH));
		canvas.addComponent(new GridNavThumb(GridNavThumb.SOUTH));
		canvas.addComponent(new GridNavThumb(GridNavThumb.EAST));
		canvas.addComponent(new GridNavThumb(GridNavThumb.WEST));
		setupMenu();
		canvas.addCalicoActionListener(this);
		inputHandler = new MouseInputHandler(canvas);
		canvas.addInputEventListener(inputHandler);
		
	}

	@Override
	public boolean updateBounds(int width, int height) {		
		if (setBounds(new Rectangle(0, 0, width, height)))
			return true;
		return false;
	}

	public void fireCalicoListener(CalicoActionEvent vbe) {
		if (vbe.getAction() instanceof BGColorChanged) {
			BGColorChanged action = (BGColorChanged)vbe.getAction();
			CGCell cell = getCellByPosition(action.cellPosition);
			cell.model.bgColor = action.color;
			refreshCell(cell);
		}
		
		else if (vbe.getAction() instanceof GridCellChanged) {
			GridCellChanged action = (GridCellChanged)vbe.getAction();
			CGCell origin = getCellByPosition(action.getOrigin());
			//CGCell dest = getCellByPosition(action.getDestination());
			setCell(action.getDestination(), origin.getImage(), origin.getModel(), origin.inhabitants);
			setCell(action.getOrigin(), null, null, 0);
		}
		
	}
	
}