package ics.uci.edu.Calico.UI.components;

import ics.uci.edu.Calico.UI.Calico;
import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.UI.components.CalicoGrid.CGCell;
import ics.uci.edu.Calico.UI.components.palettes.Palettes;
import ics.uci.edu.Calico.models.ObjectHandlerModel;
import ics.uci.edu.Calico.models.CalicoLogger;
import ics.uci.edu.Calico.models.Snapshot;
import ics.uci.edu.Calico.networking.Networking;
import ics.uci.edu.Calico.networking.SerializedCell;


import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.RenderedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.SimpleTimeZone;
import java.util.TimeZone;

import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JSeparator;

import java.awt.Image;

public class Menu extends JMenuBar{
	
	private static final long serialVersionUID = 1L;
	
	protected JMenu file = new JMenu("File");
	protected JMenu tools = new JMenu("Tools");  
	protected JMenu palmenu = new JMenu("Palettes");  
	protected JCheckBoxMenuItem logging = new JCheckBoxMenuItem("Logging");; //need to have a pointer to this to change label
	
	private CalicoCanvas canvasToBeSaved;
	private Component saveComponent;
	private CalicoCanvas loadComponent;
	private ObjectHandlerModel loadedCanvas;
	private Networking client;
	private CalicoLogger logger;
	private Calico calico;
	private String defaultDir;
	
	public Menu() {
		super();
		// Create a menu
	    add(file);
	    add(tools);
	    add(palmenu);
	    
	    
	    defaultDir = new File("dummy.dum").getAbsolutePath();
	    defaultDir = defaultDir.substring(0, defaultDir.length()-9);
	}
	
	public void removeAll(){
		file.removeAll();
		tools.removeAll();
		palmenu.removeAll();
	}
	
	public void addFileBreak(){
		 file.add(new JSeparator());
	}
	
	public void addToolBreak(){
		tools.add(new JSeparator());
	}
	
	// PALETTE STUFF
	public void addManagePalettes()
	{
		JMenuItem manage = new JMenuItem("Manage");
		
		manage.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//saveGrid();
				System.out.println("DEBUG: CLICK MANAGE PALETTES MENU");
			}
		});
		palmenu.add(manage);
	}
	
	public void addAddPalettes()
	{
		JMenuItem add = new JMenuItem("Add");
		
		add.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//saveGrid();
				System.out.println("DEBUG: CLICK ADD PALETTES MENU");
			}
		});
		palmenu.add(add);
	}
	
	public void addLoadPalette()
	{
		JMenuItem loadPalette = new JMenuItem("Load Palette");
		
		loadPalette.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				for (CalicoComponent cc : loadComponent.getCalicoComponents()) {
					if (cc instanceof Palettes) {

						// Deserialize from a file
						String filename = File.separator+"pal";
						JFileChooser fc = new JFileChooser(new File(filename));
					    if (defaultDir.length() > 0)
					    	fc.setCurrentDirectory(new File(defaultDir));
						fc.addChoosableFileFilter(new MyPaletteFilter());

						// Show open dialog; this method does not return until the dialog is closed
						fc.showOpenDialog(loadComponent);
						File selFile = fc.getSelectedFile();
						if (selFile == null)
							return;
						((Palettes)cc).loadPaletteContent(selFile.getAbsolutePath());

					}
				}
				
			}
			
		});
		palmenu.add(loadPalette);
	}
	
	public void addSavePalette()
	{
		JMenuItem savePalette = new JMenuItem("Save Palette");
		
		savePalette.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				for (CalicoComponent cc : loadComponent.getCalicoComponents()) {
					if (cc instanceof Palettes) {
						String filename = File.separator+".vbc";
						System.out.println(filename);
						String day = "";
						JFileChooser fc = new JFileChooser(new File(filename));
					    if (defaultDir.length() > 0)
					    	fc.setCurrentDirectory(new File(defaultDir));
						fc.addChoosableFileFilter(new MyPaletteFilter());
						// Show open dialog; this method does not return until the dialog is closed
						fc.showSaveDialog(saveComponent);
						File selFile = fc.getSelectedFile();
						if (selFile == null)
							return;
						((Palettes)cc).savePaletteContent(selFile.getAbsolutePath());
					}
				}
				
			}
			
		});
		palmenu.add(savePalette);
	}
	
	class MyPaletteFilter extends javax.swing.filechooser.FileFilter {
        public boolean accept(File file) {
            String filename = file.getName();
            return filename.endsWith(".pal");
        }
        public String getDescription() {
            return "*.pal";
        }
    }
	
	// END PALETTE STUFF
	
	public void addSaveOption(Component com, CalicoCanvas grid){
		//gridToBeSaved = grid;
		saveComponent = com;
		  JMenuItem save = new JMenuItem("Save Grid");
		    save.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					saveGrid();
				}
		    });
		    file.add(save);
	}
	
	public void addSaveAllAsImageOption(CalicoGrid grid) {
		gridToBeSaved = grid;
		JMenuItem saveImages = new JMenuItem("Save Grid as Images");
		saveImages.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveGridAsImages();
			}
		});
		file.add(saveImages);
	}
	
	private void saveGridAsImages() {
		File dir = new File(getTime());
		boolean success = dir.mkdir();
		if (!success) {
			//alert the user
		}
		else {
			ArrayList<Image> images = new ArrayList<Image>();
			CGCell tempCell;
			ObjectHandlerModel tempModel;
			CalicoCanvas tempCanvas = new CalicoCanvas();
			for (int i = 0; i < gridToBeSaved.getChildrenCount(); i++) {
				if (gridToBeSaved.getChild(i) instanceof CGCell &&
						((CGCell)gridToBeSaved.getChild(i)).model != null &&
						((CGCell)gridToBeSaved.getChild(i)).model.objectIdTicker > 0) {
					
					tempCell = (CGCell)gridToBeSaved.getChild(i);
					tempModel = tempCell.getModel();
					tempCanvas.setBounds(loadComponent.getBounds());
					tempCanvas.loadModel(tempModel);
					Image tempIm = tempCanvas.getImage(tempModel.bgColor);
					images.add(tempIm);
				}
			}
			try {
				File imageFile;
		        for (int i  = 0; i < images.size(); i++) {
		        	imageFile = new File(dir.getPath() + File.separator + "image" + (i+1) + ".png");
		        	ImageIO.write((RenderedImage) images.get(i), "png", imageFile);
		        }
		        JOptionPane.showMessageDialog(loadComponent, 
		        		"Grid Images successfully saved to the directory " + dir.getAbsolutePath());
		    } catch (IOException e) {
		    	System.err.println(e.getMessage());
		    }
			
		}
		
	}	
	
	public void saveCanvas(){
		ObjectHandlerModel canvasModel;
		canvasModel = canvasToBeSaved.getModel();
		
		try {
	        // Serialize to a file
			String filename = File.separator+".cg";
			System.out.println(filename);
			String day = "";
		    JFileChooser fc = new JFileChooser(new File(filename));
		    fc.setCurrentDirectory(null);
		    fc.addChoosableFileFilter(new MyFilter());
		    // Show open dialog; this method does not return until the dialog is closed
		    fc.showSaveDialog(saveComponent);
		    File selFile = fc.getSelectedFile();
		    if (selFile == null)
		    	return;
	        ObjectOutput out = new ObjectOutputStream(new FileOutputStream(selFile));
	        out.writeObject(canvasModel);
	        out.close();
	    } catch (IOException e) {
	    	System.err.println(e.getMessage());
	    }
	}
	
	
	/*
	public void addLoadOption(CalicoCanvas com){
		loadComponent = com;
		   JMenuItem load = new JMenuItem("Load Grid");
		    load.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					loadGrid();				
				}
		    });
		    file.add(load);
	}*/
	
	public void loadCanvas(){
		try {
	        // Deserialize from a file
		    String filename = File.separator+"vbc";
		    JFileChooser fc = new JFileChooser(new File(filename));
		    if (defaultDir.length() > 0)
		    	fc.setCurrentDirectory(new File(defaultDir));
		    fc.addChoosableFileFilter(new MyFilter());
		    
		    // Show open dialog; this method does not return until the dialog is closed
		    fc.showOpenDialog(loadComponent);
		    File selFile = fc.getSelectedFile();
		    if (selFile == null)
		    	return;
	        ObjectInputStream in = new ObjectInputStream(new FileInputStream(selFile));
	        // Deserialize the object
	        loadedCanvas = (ObjectHandlerModel) in.readObject();
	        in.close();
	        loadComponent.loadModel(loadComponent.getModel());
	        
	    } catch (ClassNotFoundException e) {
	    	System.err.println(e.getMessage());
	    } catch (IOException e) {
	    	System.err.println(e.getMessage());
	    }
	}
	
	  public void addExitOption(CalicoLogger log){
		  	logger = log;
	    	JMenuItem exit = new JMenuItem("Exit");
		    exit.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					exit();	
				}
		    });
		    file.add(exit);
	    }
	  
	  public void exit(){
		  logger.finalize();
		  System.exit(0);
	  }
	
	  public ObjectHandlerModel getLoadedCanvas(){
		  return loadedCanvas;
	  }
	  
	  public void addConnect(Networking client){
		  this.client = client;
		  JMenuItem addConnect = new JMenuItem("Connect");
		  addConnect.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					drawConnectFrame();		
				}
		    });
		    tools.add(addConnect);
	  }
	  
	  private void drawConnectFrame(){
		  client.startClient();
		  client.drawConnectionFrame();
	  }
	  
	  public void addDisconnect(Networking client){
		  this.client = client;
		  JMenuItem addDisconnect = new JMenuItem("Disconnect");
		  addDisconnect.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					addDisconnectFrame();		
				}
		    });
		    tools.add(addDisconnect);
	  }
	  
	  private void addDisconnectFrame(){
		  client.disconnect();
	  }
	  
	  public void addStartServer(Networking client){
		  this.client = client;
		  JMenuItem startServer = new JMenuItem("Start Server");
		  startServer.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					startServer();
					
				}
		    });
		  tools.add(startServer);
	  }
	  
	  private void startServer(){
		  client.startServer();
	  }
	  
	  private void message(String message){
		   JFrame errorFrame = new JFrame();
		   JOptionPane.showMessageDialog(errorFrame,
				    message,
				    "Server Error",
				    JOptionPane.INFORMATION_MESSAGE);
		   
	   }
	  
	class MyFilter extends javax.swing.filechooser.FileFilter {
        public boolean accept(File file) {
            String filename = file.getName();
            return filename.endsWith(".cg");//Calico Grid
        }
        public String getDescription() {
            return "*.cg";
        }
    }
	
	public void addLoggingOption(Calico cal){
			logging.setSelected(cal.LOGGING);
			logger = cal.getLogger();
			logging.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					toggleLogging();				
				}
		    });
		    file.add(logging);
	}
	
	public void toggleLogging(){
		if(calico.LOGGING){
			calico.getLogger().closeStream();
			calico.LOGGING = false;
		}
		else{
			calico.initializeLogger();
			calico.LOGGING = true;
		}
		logging.setSelected(calico.LOGGING);
	}
	
	/*
	 * GRID STUFF
	 */
	private CalicoGrid gridToBeSaved;
	
	public void addRefreshOption(CalicoGrid grid){
		gridToBeSaved = grid;
		JMenuItem refreshGrid = new JMenuItem("Refresh Grid");
	    refreshGrid.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				refreshGrid();
			}
	    });
	    file.add(refreshGrid);
	}
	
	private void refreshGrid(){
		gridToBeSaved.calicoInstance.requestGridUpdate();
		System.out.println();
	}
	
	
	public void addSaveGridOption(CalicoGrid grid){
		gridToBeSaved = grid;
		  JMenuItem saveGrid = new JMenuItem("Save Grid");
		    saveGrid.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					saveGrid();
				}
		    });
		    file.add(saveGrid);
	}
	
	public void saveGrid() {
		// Serialize to a file
		String filename = File.separator+"cg";
	    JFileChooser fc = new JFileChooser(new File(filename));
	    fc.setSelectedFile(new File(Calico.getVersion() + "_" + getTime() + ".cg"));
	    if (defaultDir.length() > 0)
	    	fc.setCurrentDirectory(new File(defaultDir));
	    fc.addChoosableFileFilter(new MyFilter());
	    
	    // Show open dialog; this method does not return until the dialog is closed
	    fc.showSaveDialog(gridToBeSaved.frame);
	    File selFile = fc.getSelectedFile();
	    String path = selFile.getPath();
	    String name = selFile.getName();
	    defaultDir = selFile.getPath().substring(0, path.length() - name.length());
	    saveGrid(selFile);
	}
	
	
	public void saveGrid(File file) {
		ArrayList<SerializedCell> cells = new ArrayList<SerializedCell>();
		
		
		for (int i = 0; i < gridToBeSaved.getChildrenCount(); i++) {
			if (gridToBeSaved.getChild(i) instanceof CGCell &&
					((CGCell)gridToBeSaved.getChild(i)).model != null &&
					((CGCell)gridToBeSaved.getChild(i)).model.objectIdTicker > 0) {
				cells.add(new SerializedCell((CGCell)gridToBeSaved.getChild(i)));
			}
		}
		
		try {
	        
		    if (file == null)
		    	return;
	        ObjectOutput out = new ObjectOutputStream(new FileOutputStream(file));
	        out.writeObject(cells);
	        out.close();
	    } catch (IOException e) {
	    	System.err.println(e.getMessage());
	    }
	}
	
	
	
	public void addLoadGridOption(CalicoCanvas com){
		loadComponent = com;
		  JMenuItem loadGrid = new JMenuItem("Load Grid");
		    loadGrid.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					loadGrid();
				}
		    });
		    file.add(loadGrid);
	}
	
	
	private void loadGrid() {
		
		try {
	        // Deserialize from a file
		    String filename = File.separator+"cg";
		    JFileChooser fc = new JFileChooser(new File(filename));
		    
		    if (defaultDir.length() > 0)
		    	fc.setCurrentDirectory(new File(defaultDir));
		    fc.addChoosableFileFilter(new MyFilter());
		    
		    // Show open dialog; this method does not return until the dialog is closed
		    fc.showOpenDialog(gridToBeSaved.frame);
		    File selFile = fc.getSelectedFile();
		    if (selFile == null)
		    	return;
		    
		    String path = selFile.getPath();
		    String name = selFile.getName();
		    defaultDir = selFile.getPath().substring(0, path.length() - name.length());
		    
		    //TODO this is how to load!
	        ObjectInputStream in = new ObjectInputStream(new FileInputStream(selFile));
	        // Deserialize the object
	        ArrayList<SerializedCell> cells = (ArrayList<SerializedCell>)in.readObject();
	        in.close();
	        
	        gridToBeSaved.clear();
	        CalicoCanvas canvas = new CalicoCanvas();
	        canvas.setBounds(loadComponent.getBounds());
	        for (SerializedCell cell : cells) {
	        	canvas.loadModel(cell.model);
	        	gridToBeSaved.setCell(cell.position, canvas.getImage(cell.model.bgColor), cell.model, cell.inhabitants);
	        }
	        System.out.println("Finished Loading Grid");
	        gridToBeSaved.repaint();	        
	    } catch (ClassNotFoundException e) {
	    	System.err.println(e.getMessage());
	    } catch (IOException e) {
	    	System.err.println(e.getMessage());
	    }
		
	}

	class MyGridFilter extends javax.swing.filechooser.FileFilter {
        public boolean accept(File file) {
            String filename = file.getName();
            return filename.endsWith(".vbg");
        }
        public String getDescription() {
            return "*.VBG";
        }
    }
	
	public static void saveImage(RenderedImage img, String fileName) {
		// Write generated image to a file
	    try {
	        // Save as JPEG
	        File file = new File(fileName);
	        ImageIO.write(img, "jpg", file);
	    } catch (IOException e) {
	    }
	}
	
	public static String getTime() {
			
		  //Date timestamp = new Date(System.currentTimeMillis());
		  String[] ids = TimeZone.getAvailableIDs(-8 * 60 * 60 * 1000);
		  SimpleTimeZone pdt = new SimpleTimeZone(-8 * 60 * 60 * 1000, ids[0]);

		  // set up rules for daylight savings time
		  pdt.setStartRule(Calendar.MARCH, 1, Calendar.SUNDAY, 2 * 60 * 60 * 1000);
		  pdt.setEndRule(Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * 60 * 60 * 1000);
		  
		  Calendar calendar = new GregorianCalendar(pdt);
		  Date currentTime = new Date();
		  calendar.setTime(currentTime);

		  int realMonth = calendar.get(Calendar.MONTH) + 1;
		  return new String(
				  Calico.getVersion() + "_"+
				  realMonth+"-"+
				  calendar.get(Calendar.DAY_OF_MONTH)+"-"+
				  getSmallYear(calendar.get(Calendar.YEAR))+"_"+
				  get12HourTime(calendar.get(Calendar.HOUR_OF_DAY), calendar.get(Calendar.MINUTE), calendar.get(Calendar.SECOND))
		  );
		
	}
	
	public static String get12HourTime(int hour, int minute, int second){
		String AMPM;
		String m;
		String s;
		Integer h = new Integer(hour);
		
		if(hour > 11){
			AMPM = "PM";}
		else{
			AMPM = "AM";}		
		
		if(hour == 0){
			h = new Integer(12);}
		if(hour > 12){
			h = new Integer(hour - 12);}
		
		if(minute < 10){
			m = "0" + new Integer(minute).toString();}
		else{
			m = new Integer(minute).toString();}
		
		s = (second < 10)?"0":"" + new Integer(second).toString();
		
		return h+"-"+m+"-"+s+AMPM;
	}
	
	public static String getSmallYear(int year){
		Integer i = new Integer(year);
		return i.toString().substring(2);
	}
	
}



