package ics.uci.edu.Calico.UI.components;

import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.models.ObjectHandlerModel;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolo.util.PPaintContext;

/**
 * Selector is a component that takes over the whole screen. It's
 * basically for showing two different images, and returning the image
 * that the user selects. Created with loading buckets in mind.
 * 
 * Note: This class is defunct, this functionality may be later implemented
 * with the grid.
 * 
 * @author nmangano
 *
 */
public class Selector extends PNode {

	private Image img1, img2;
	private int windowW, windowH;
	private CalicoCanvas canvas;
	private ObjectHandlerModel model;
	private PBounds imageBounds;
	

	static int MOUSE_OUTSIDE = 0;
	static int MOUSE_IMG1 = 1;
	static int MOUSE_IMG2 = 2;
	
	private int mouseLocation = MOUSE_OUTSIDE;

	public Selector(Image i1, Image i2, ObjectHandlerModel m, int w, int h, CalicoCanvas c) {
		img1 = i1;
		img2 = i2;
		windowW = w;
		windowH = h;
		canvas = c;
		model = m;
		
		this.addInputEventListener(new SelectorInputHandler());
	}
	
	public boolean setBounds() {
		double localWidth = img1.getWidth(null) + img2.getWidth(null);
		//the height will be the taller of the two
		double localHeight = (img1.getHeight(null) > img2.getHeight(null))?img1.getHeight(null):img2.getHeight(null);
		
		//The one with the highest ratio will be the one to be downsized
		if (localWidth/windowW > localHeight/windowH) {
			if (localWidth > windowW * .8) {
				double scale = (windowW*.8)/(img1.getWidth(null)+img2.getWidth(null));
				img1 = img1.getScaledInstance(new Double(img1.getWidth(null)*scale).intValue(), 
						new Double(img1.getHeight(null)*scale).intValue(), Image.SCALE_SMOOTH);
				img2 = img2.getScaledInstance(new Double(img2.getWidth(null)*scale).intValue(), 
						new Double(img2.getHeight(null)*scale).intValue(), Image.SCALE_SMOOTH);
			}
		} 
		else {
			if (localHeight > windowH * .8) {
				double scale = (windowH*.8)/(img1.getHeight(null)+img2.getHeight(null));
				img1 = img1.getScaledInstance(new Double(img1.getWidth(null)*scale).intValue(), 
						new Double(img1.getHeight(null)*scale).intValue(), Image.SCALE_SMOOTH);
				img2 = img2.getScaledInstance(new Double(img2.getWidth(null)*scale).intValue(), 
						new Double(img2.getHeight(null)*scale).intValue(), Image.SCALE_SMOOTH);
			}
		}
			
		//compute it again in case they were changed
		localWidth = img1.getWidth(null) + img2.getWidth(null);
		localHeight = (img1.getHeight(null) > img2.getHeight(null))?img1.getHeight(null):img2.getHeight(null);
		
		double localX = windowW/2 - localWidth/2;
		double localY = windowH/2 - localHeight/2;
		
		imageBounds = new PBounds(localX, localY, localWidth + 2, localHeight);
		if (super.setBounds(new Double(windowW * .05).intValue(), new Double(windowH * .05).intValue(), 
				new Double(windowW * .9).intValue(), new Double(windowH * .9).intValue()))
			//do nothing
			return true;
		else
			return false;
		
	}
	
	public void paint(PPaintContext aPaintContext) {
		Graphics2D g2 = aPaintContext.getGraphics(); 
		g2.setPaint(getPaint());
		
		g2.setColor(new Color(0,0,0, 200));
		g2.fillRect(new Double(getBounds().x).intValue(), new Double(getBounds().y).intValue(), 
				new Double(getBounds().width).intValue(), new Double(getBounds().height).intValue());
		
		
		g2.setColor(Color.WHITE);
		g2.drawString("Which version would you like to continue with?", 
				new Double(getBounds().x).intValue() + 5, new Double(getBounds().y + 10).intValue());
		g2.drawString("Current version", new Double(imageBounds.x).intValue(), 
				new Double(imageBounds.y - 3).intValue());
		g2.drawString("Bucket", new Double(imageBounds.x + img1.getWidth(null)).intValue() + 1,
				new Double(imageBounds.y - 3).intValue());
		
		
		
		/*
		JLabel title = new JLabel("Please select a version");
		title.setBounds(new Double(getBounds().x).intValue(),new Double(getBounds().y).intValue(), 
				new Double(getBounds().width).intValue(), 10);
		title.setLocation(new Double(getBounds().x).intValue(),new Double(getBounds().y).intValue());
		title.setForeground(Color.black);
		//title.setHorizontalTextPosition(JLabel.CENTER);
		title.paint(g2);		
		*/
		
		g2.setColor(Color.WHITE);
		g2.fillRect(new Double(imageBounds.x).intValue(), new Double(imageBounds.y).intValue(), 
				img1.getWidth(null), img1.getHeight(null));
		g2.drawImage(img1, new Double(imageBounds.x).intValue(), new Double(imageBounds.y).intValue(), null);
		g2.fillRect(new Double(imageBounds.x + img1.getWidth(null)).intValue() + 1,new Double(imageBounds.y).intValue(), 
				img2.getWidth(null), img2.getHeight(null));
		g2.drawImage(img2, new Double(imageBounds.x + img1.getWidth(null)).intValue() + 1, 
				new Double(imageBounds.y).intValue(), null);
		
		//Draws a frame around each Image. If the mouse is inside the region of that frame, it will be highlighted Blue
		g2.setColor((mouseLocation == MOUSE_IMG1)?Color.BLUE:Color.BLACK);
		g2.setStroke((mouseLocation == MOUSE_IMG1)?new BasicStroke(2.0f):new BasicStroke(1.0f));
		g2.drawRect(new Double(imageBounds.x).intValue(), new Double(imageBounds.y).intValue(), 
					img1.getWidth(null), img1.getHeight(null));
		g2.setColor((mouseLocation == MOUSE_IMG2)?Color.BLUE:Color.BLACK);
		g2.setStroke((mouseLocation == MOUSE_IMG2)?new BasicStroke(2.0f):new BasicStroke(1.0f));
		g2.drawRect(new Double(imageBounds.x + img1.getWidth(null)).intValue() + 1, 
				new Double(imageBounds.y).intValue(), img2.getWidth(null), img2.getHeight(null));
		g2.setStroke(new BasicStroke(1.0f));
		g2.setColor(Color.BLACK);
	}
	
	private class SelectorInputHandler extends PBasicInputEventHandler {

		Rectangle i1, i2;
		
		public SelectorInputHandler() {

		}
		
		public void mouseMoved(PInputEvent e) {
			if (i1 == null || i2 == null) {
				i1 = new Rectangle(new Double(imageBounds.x).intValue(), 
						new Double(imageBounds.y).intValue(), img1.getWidth(null), img1.getHeight(null));
				i2 = new Rectangle(new Double(imageBounds.x + img1.getWidth(null)).intValue(), 
						new Double(imageBounds.y).intValue(), img2.getWidth(null), img2.getHeight(null));
			}
			if (i1.contains(e.getCanvasPosition()))
				mouseLocation = MOUSE_IMG1;
			else if (i2.contains(e.getCanvasPosition()))
				mouseLocation = MOUSE_IMG2;
			else
				mouseLocation = MOUSE_OUTSIDE;
			repaint();
		}
		
		public void mouseClicked(PInputEvent e) {
			if (i1 == null || i2 == null)
				return;
			if (i1.contains(e.getCanvasPosition()))
				;
			else if (i2.contains(e.getCanvasPosition()))
				canvas.loadModel(model);
			getParent().removeChild(Selector.this);
		
		}
		
		public void mouseExited(PInputEvent e) {
			mouseLocation = MOUSE_OUTSIDE;
		}
	}
}
