package ics.uci.edu.Calico.UI.components;

import ics.uci.edu.Calico.UI.CalicoCanvas;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

import edu.umd.cs.piccolo.util.PPaintContext;

/**
 * Zoom is just the UI circle for zooming. The actual zooming interaction is
 * handled by MouseInputHandler and the zoom framework from Piccolo.
 * 
 * @author nmangano
 *
 */
public class Zoom extends CalicoComponent {
	
	int x = 0,y = 0;
	int centerX, centerY;
	double zoomPercent = 1.0;
	
	public Zoom() {

	}
	
	public void paint(PPaintContext paintContext) {
		Graphics2D g = paintContext.getGraphics();
		g.setPaint(getPaint());
		
		int xVal = new Double(getX()).intValue();
		int yVal = new Double(getY()).intValue();
		
		g.setColor(Color.white);
		g.fillOval(xVal, yVal, new Double(getWidth()).intValue()-10, new Double(getHeight()).intValue()-10);
		
		g.setStroke(new BasicStroke(1.5f));
		g.setColor(Color.black);
		g.drawOval(xVal, yVal, new Double(getWidth()).intValue()-10, new Double(getHeight()).intValue()-10);
		
		Line2D line = new Line2D.Double(0, 0, 0, 30);	
		double delta = (x == 0 && y == 0)?0:calcAngle(centerX, centerY, x, y);
		
		AffineTransform at = new AffineTransform();
		at.translate(getX() + getWidth()/2 - 5, getY() + getHeight()/2 - 5);
		at.rotate(delta);
		Shape li = at.createTransformedShape(line);
		g.draw(li);
		
		g.setColor(Color.white);
		g.fillOval(xVal+10, yVal+10, new Double(getWidth()).intValue()-30, new Double(getHeight()).intValue()-30);
		g.setColor(Color.black);
		g.drawString((int)(zoomPercent*100) + "%", xVal+10, yVal+30);
	}
	
	public static double calcAngle(float x1, float y1, float x2, float y2)
    {
        float dx = x2-x1;
        float dy = y2-y1;
        double angle=0.0d;
 
        // Calculate angle
        if (dx == 0.0)
        {
            if (dy == 0.0)
                angle = 0.0;
            else if (dy > 0.0)
                angle = Math.PI / 2.0;
            else
                angle = Math.PI * 3.0 / 2.0;
        }
        else if (dy == 0.0)
        {
            if  (dx > 0.0)
                angle = 0.0;
            else
                angle = Math.PI;
        }
        else
        {
            if  (dx < 0.0)
                angle = Math.atan(dy/dx) + Math.PI;
            else if (dy < 0.0)
                angle = Math.atan(dy/dx) + (2*Math.PI);
            else
                angle = Math.atan(dy/dx);
        }
 
        // Convert to degrees
        //angle = angle * 180 / Math.PI;
 
        // Return
        return angle;
    }
	
	public void setZoomScale(double z) {
		zoomPercent = z;
	}
	
	public double getZoomScale() {
		return zoomPercent;
	}
	
	public boolean setBounds(double x, double y, double width, double height) {
		
		centerX = new Double(x + width/2).intValue();
		centerY = new Double(y + height/2).intValue();
		
		if (super.setBounds(x, y, width, height)) {
			return true;
		}
		else
			return false;
	}
	
	
	/**
	 * Sets the position for the indictator line in the GUI.
	 * @param p The point the zoom line will point to.
	 */
	public void setPoint(Point2D p) {
		x = new Double(p.getX()).intValue();
		y = new Double(p.getY()).intValue();
		repaint();
	}
	
	public boolean contains(Point2D p) {
		return getBounds().contains(p);
	}

	@Override
	public boolean updateBounds(int width, int height) {
		if (setBounds(width - 50 - 50, 105, 60,60))
			return true;
		return false;
		
	}

	@Override
	public void registerCanvas(CalicoCanvas canvas) {
		//do nothing
	}
	
}
