package ics.uci.edu.Calico.UI.components;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;

import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PPaintContext;
import ics.uci.edu.Calico.UI.CalicoCanvas;

public class ZoomSlider extends CalicoComponent {

	CalicoCanvas canvas;
	int width = 40, height = 200;
	Polygon slider;
	int percentOnSlider, actualPercent;
	
	int lastPercent = 0;
	
	int[] zoomValues = {12, 25, 50, 100, 200, 400, 800};
	
	public ZoomSlider() {
		slider = new Polygon();
		addInputEventListener(new ZoomSliderInputHandler());
	}
	
	public void paint(PPaintContext ppc) {
		Graphics2D g = ppc.getGraphics();
		
		g.setColor(Color.black);
		g.setStroke(new BasicStroke(1.0f));
		Rectangle2D rect = getBounds();
		g.drawRect(new Double(rect.getX() + rect.getWidth() * .25).intValue(), new Double(rect.getY()).intValue(), 
				new Double(rect.getWidth() * .75).intValue(), new Double(rect.getHeight()).intValue());
		//draw dashes
		int yOffset;
		
		for (int i = 0; i < 200; i += 25) {
			yOffset = new Double(rect.getY() + i/200.0 * rect.getHeight() + rect.getHeight()/8).intValue();
			g.drawLine(new Double(rect.getX() + rect.getWidth() * .25).intValue(), yOffset,
					new Double(rect.getX() + rect.getWidth() * .25 + 5).intValue(), yOffset);
			if (i/25 > -1 && i/25 < 7)
				g.drawString(zoomValues[6 - i/25] + "", new Double(rect.getX() + rect.getWidth() * .25 + 6).intValue(), 
					new Double(yOffset + 4).intValue());
		}
		
		setSlider(percentOnSlider);
		g.setColor(new Color(167, 167, 138));
		g.fillPolygon(slider);
		g.setColor(Color.black);
		g.drawPolygon(slider);
		
	}
	
	@Override
	public void registerCanvas(CalicoCanvas c) {
		canvas = c;
		setPercent(100);
	}

	@Override
	public boolean updateBounds(int w, int h) {
		if (setBounds(w - width, h - height, width, height))  {
			
			return true;
		}
		return false;
	}
	
	/*
	 * The user can zoom between 25%, and 200%
	 * so, p can be between 0 and 200.
	 */
	public void setSlider(int p) {
		int xOffset = 0, yOffset = 0;
		if (p < 0 || p > 200)
			return;
		
		if(p==lastPercent)
			return;
		
		lastPercent = p;
		
		slider.reset();
		slider.addPoint(0, 0);
		slider.addPoint(0, 20);
		slider.addPoint(20, 20);
		slider.addPoint(30, 10);
		slider.addPoint(20, 0);
		
		xOffset = new Double(getBounds().getX()).intValue();
		yOffset = new Double(getBounds().getY() + getBounds().getHeight()).intValue();
		
		yOffset -= new Double(percentOnSlider/200.0 * getBounds().getHeight()).intValue() + 10;
		
		slider.translate(xOffset, yOffset);
		repaint();
	}
	
	/**
	 * This just centers the pointer around the y coordinate of input (x,y).
	 * @param x
	 * @param y
	 */
	private void setSlider(int x, int y) {
		int xOffset = 0, yOffset = 0;
		if (y < getBounds().getY() || y > getBounds().getY() + getBounds().getHeight())
			return;
		
		
		slider.reset();
		slider.addPoint(0, 0);
		slider.addPoint(0, 20);
		slider.addPoint(20, 20);
		slider.addPoint(30, 10);
		slider.addPoint(20, 0);
		
		xOffset = new Double(getBounds().getX()).intValue();
		yOffset = y - 10;
		
		//slider.translate(20, 20);
		//updateBounds(new Double(canvas.getBounds().getWidth()).intValue(), new Double(canvas.getBounds().getHeight()).intValue());
	}
	
	public void setPercent(int p) {
		percentOnSlider = p;
	}
	
	private int getPercent(int x, int y) {
		int xOffset = 0, yOffset = 0;
		if (y < getBounds().getY() || y > getBounds().getY() + getBounds().getHeight())
			return -1;
		
		int percen = new Double(((getBounds().getY() + getBounds().getHeight()) - y)/getBounds().getHeight() * 200).intValue();
		return percen;
	}
	
	private void snap() {
		//if (y < getBounds().getY() || y > getBounds().getY() + getBounds().getHeight())
		//	return;
		
		//snap slider
		if (percentOnSlider < avg(25, 50)) {
			percentOnSlider = 25;
			actualPercent = zoomValues[0];
		}
		else if (percentOnSlider < avg (50, 75)) {
			percentOnSlider = 50;
			actualPercent = zoomValues[1];
		}
		else if (percentOnSlider < avg(75, 100)) {
			percentOnSlider = 75;
			actualPercent = zoomValues[2];
		}
		else if (percentOnSlider < avg(100, 125)) {
			percentOnSlider = 100;
			actualPercent = zoomValues[3];
		}
		else if (percentOnSlider < avg(125, 150)) {
			percentOnSlider = 125;
			actualPercent = zoomValues[4];
		}
		else if (percentOnSlider < avg(150, 175)) {
			percentOnSlider = 150;
			actualPercent = zoomValues[5];
		}
		else if (percentOnSlider < 200) {
			percentOnSlider = 175;
			actualPercent = zoomValues[6];
		}
		
		
		setSlider(percentOnSlider);
	}
	
	private int avg(int a, int b) {
		return (a + b) / 2;
	}
	
	private class ZoomSliderInputHandler extends PBasicInputEventHandler {
		
		private int initY;
		boolean grabbed = false;
		
		public void mousePressed(PInputEvent e) {
			if (slider.contains(e.getCanvasPosition())) {
				grabbed = true;
				int per = getPercent(new Double(e.getCanvasPosition().getX()).intValue(), new Double(e.getCanvasPosition().getY()).intValue()); 
				setPercent(per);
				translate(0,0);
			}
		}
		
		public void mouseDragged(PInputEvent e) {
			if (grabbed) {
				int per = getPercent(new Double(e.getCanvasPosition().getX()).intValue(), new Double(e.getCanvasPosition().getY()).intValue());
				setPercent(per);
				repaint();
			}
			
		}
		
		public void mouseReleased(PInputEvent e) {
			if (grabbed) {
				snap();
				double w = canvas.getBounds().getWidth(),
					h = canvas.getBounds().getHeight();
				double inverseScale = 1 / canvas.getLayer().getCamera(0).getViewScale();
				Rectangle vbounds = canvas.getLayer().getCamera(0).getViewBounds().getBounds(); 
				canvas.getLayer().getCamera(0).scaleViewAboutPoint(inverseScale * actualPercent / 100.0, 
						vbounds.getX() + vbounds.getWidth()/2, vbounds.getY() + vbounds.getHeight()/2);
				
				
				//e.getCamera().translateView
				//canvas.getLayer().getCamera(0).scale
				
				//e.getCamera().setScale(percent / 100.0);
				//System.out.println("ViewScale: " + e.getCamera().getViewScale());
				
				//e.getCamera().scaleView(arg0)
				//e.getCamera().scaleViewAboutPoint(arg0, arg1, arg2)
			}
			grabbed = false;
		}
	}

}
