package ics.uci.edu.Calico.UI.components.palettes;

import ics.uci.edu.Calico.Structures.*;
import ics.uci.edu.Calico.UI.*;
import ics.uci.edu.Calico.UI.components.*;
import ics.uci.edu.Calico.UI.components.palettes.*;
import ics.uci.edu.Calico.models.*;
import ics.uci.edu.Calico.models.actions.*;

import java.io.*;
import java.net.*;
import java.awt.*;
import javax.swing.ImageIcon;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.*;

import edu.umd.cs.piccolo.PCanvas;
import edu.umd.cs.piccolo.util.PPaintContext;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.util.PPaintContext;


import java.nio.Buffer;
import java.nio.ByteBuffer;

/**
 * A customizable bar that can save and load figures.
 *
 * 
 * @author nmangano
 *
 */
public class Palettes extends CalicoComponent implements CalicoActionListener {

	public static int PULL_HANDLE = -2;
	
	int row, col;
	CellContents[] cellContents;
	Image[] cellImages;
	CalicoCanvas canvas;
	public boolean isInHiddenMode = true;
	private Rectangle2D showHideBar;
	
	int biggestWindow = 0;
	int nameWidth = 12;

	HashMap<String,ByteFaker> cellhash = new HashMap<String,ByteFaker>();

	
	boolean isServerActive = true;
	
	
	/*
	
		PaletteItemUsed
		PaletteItemMoved
		PaletteItemSet
		PaletteSaved
		PaletteLoaded
		PaletteItemDeleted
		PaletteUnloaded

		canvas.applyAction(new PaletteItemUsed(...) )
	*/
	
	private void debug(String msg)
	{
		System.out.println("PALETTES DEBUG: "+msg);
	}
	
	public Palettes() {
		addInputEventListener(new PalettesListener());	
		row = CalicoOptions.pal_rows;
		col = CalicoOptions.pal_cols;
		
		cellContents = new CellContents[row*col];
		cellImages = new Image[row*col];
		
		// someone set us up the cell hash!
		for(int i=0;i<cellContents.length;i++)
		{
			cellhash.put( "cell"+i, ByteFaker.wrap(new byte[1]) );
			cellhash.put( "image"+i, ByteFaker.wrap(new byte[1]) );
		}



		if(CalicoOptions.pal_savetype.equalsIgnoreCase("SERVER"))
		{
			isServerActive = true;
		}
		else
		{
			isServerActive = false;
			loadPaletteContent("default.pal");
		}
		
		for(int i=0;i<cellContents.length;i++)
		{
			getCell(i);
		}

		
		
		
		//setBounds(new Rectangle2D.Double(0,0,100,200));
		setBounds(new Rectangle2D.Double(0,0,112,200));
	}

	public void loadPaletteContent(String filename)
	{
		//canvas.applyAction(new PaletteLoaded() );
		FileInputStream fos = null;
		ObjectInputStream in = null;
		try
		{
			fos = new FileInputStream(filename);
			in = new ObjectInputStream(fos);
			cellhash = (HashMap)in.readObject();
			in.close();

			for(int i=0; i<cellContents.length;i++)
			{
				try
				{
					cellContents[i] = new CellContents( (HashMap)bytesToObject(cellhash.get("cell"+i).getBytes()));
					cellImages[i] =  ((ImageIcon)bytesToObject(cellhash.get("image"+i).getBytes() )).getImage();
				}
				catch(Exception e2)
				{
					// fail
				}
			}

		}
		catch(Exception e)
		{
			e.printStackTrace();
			debug("You broke it.");
		}

	}
	public void savePaletteContent(String filename)
	{
		canvas.applyAction(new PaletteSaved() );
		FileOutputStream fos = null;
		ObjectOutputStream out = null;
		try
		{
			fos = new FileOutputStream(filename);
			out = new ObjectOutputStream(fos);
			out.writeObject( cellhash );
			out.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			debug("You broke it.");
		}
	}
	
	private byte[] makePacket(int command, int cell)
	{
		// -128 to 127, 0 to 255 
		byte[] packetData = new byte[16];
		packetData[0] = new Integer(command).byteValue();
		packetData[1] = new Integer(cell).byteValue();
		packetData[2] = new Integer(0).byteValue();
		packetData[3] = new Integer(0).byteValue();
		packetData[4] = new Integer(0).byteValue();
		packetData[5] = new Integer(0).byteValue();
		packetData[6] = new Integer(0).byteValue();
		packetData[7] = new Integer(0).byteValue();
		packetData[8] = new Integer(0).byteValue();
		packetData[9] = new Integer(0).byteValue();
		packetData[10] = new Integer(0).byteValue();
		packetData[11] = new Integer(0).byteValue();
		packetData[12] = new Integer(0).byteValue();
		packetData[13] = new Integer(0).byteValue();
		packetData[14] = new Integer(0).byteValue();
		packetData[15] = new Integer(0).byteValue();
		return packetData;
	}
	private byte[] makePacket(int command, int cell, byte[] bytes)
	{
		byte[] packetData = new byte[16+bytes.length];
		packetData[0] = new Integer(command).byteValue();
		packetData[1] = new Integer(cell).byteValue();
		packetData[2] = new Integer(0).byteValue();
		packetData[3] = new Integer(0).byteValue();
		packetData[4] = new Integer(0).byteValue();
		packetData[5] = new Integer(0).byteValue();
		packetData[6] = new Integer(0).byteValue();
		packetData[7] = new Integer(0).byteValue();
		packetData[8] = new Integer(0).byteValue();
		packetData[9] = new Integer(0).byteValue();
		packetData[10] = new Integer(0).byteValue();
		packetData[11] = new Integer(0).byteValue();
		packetData[12] = new Integer(0).byteValue();
		packetData[13] = new Integer(0).byteValue();
		packetData[14] = new Integer(0).byteValue();
		packetData[15] = new Integer(0).byteValue();
		for(int i=0;i<bytes.length;i++)
		{
			packetData[i+16] = bytes[i];
		}
		return packetData;
	}
	
	private byte[] objectToBytes(Object obj)
	{
		try
		{
			ByteArrayOutputStream fos = new ByteArrayOutputStream();
			ObjectOutputStream out = new ObjectOutputStream(fos);
			out.writeObject(obj);
			out.close();
			return fos.toByteArray();
		}
		catch(Exception e)
		{
			return new byte[1];
		}
	}
	private Object bytesToObject(byte[] bytes) throws Exception
	{
		ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
		ObjectInputStream in = new ObjectInputStream(bais);
		Object obj = in.readObject();
		in.close();
		return obj;
	}
	private void sendPacket(byte[] packetData)
	{
		try
		{
			DatagramSocket socket = new DatagramSocket(CalicoOptions.pal_port);
			InetAddress address = InetAddress.getByName(CalicoOptions.pal_server);
			DatagramPacket packet = new DatagramPacket(packetData, 0, packetData.length, address, CalicoOptions.pal_port);
			socket.send(packet);
			socket.close();
		}
		catch(Exception e)
		{
			debug("Apparently, the server has exploded. We are deeply sorry.");
		}
	}
	private byte[] sendRecvPacket(byte[] packetData)
	{
		try
		{
			DatagramSocket socket = new DatagramSocket(CalicoOptions.pal_port);
			socket.setSoTimeout(CalicoOptions.pal_timeout);
			InetAddress address = InetAddress.getByName(CalicoOptions.pal_server);
			DatagramPacket packet = new DatagramPacket(packetData, 0, packetData.length, address, CalicoOptions.pal_port);
			socket.send(packet);
			
			byte[] rdata = new byte[10240];
			DatagramPacket recv = new DatagramPacket(rdata,rdata.length);
			socket.receive(recv);
			rdata = recv.getData();
			
			socket.close();
			return rdata;
		}
		catch(SocketTimeoutException to)
		{
			isServerActive = false;
			debug("Could not connect to Palette Server.");
		}
		catch(Exception e)
		{
			debug("Apparently, the server has exploded. We are deeply sorry.");
		}
		byte[] error = new byte[1];
		error[0] = 0;
		return error;
	}
	
	private void sendCellImage(int cell)
	{
		try
		{
			byte[] data = objectToBytes( new ImageIcon(cellImages[cell]));
			cellhash.put("image"+cell, ByteFaker.wrap(data));
		
			if(isServerActive)
			{
				sendPacket(makePacket(PaletteCommand.SEND_CELL_IMAGE.ordinal(), cell, data));
			}
		}
		catch(Exception e)
		{
			debug("Whatever you did, you broke it.");
			//e.printStackTrace();
		}
	}
	
	private void sendCellContents(int cell)
	{
		try
		{
			byte[] data = objectToBytes( cellContents[cell].exportContents() );
			cellhash.put("cell"+cell, ByteFaker.wrap(data));

			if(isServerActive)
			{
				sendPacket(makePacket(PaletteCommand.SEND_CELL_CONTENTS.ordinal(), cell, data ));
			}
		}
		catch(Exception e)
		{
			debug("Whatever you did, you broke it.");
			//e.printStackTrace();
		}
	}
	
	
	private void sendCell(int cell)
	{
		sendCellImage(cell);
		sendCellContents(cell);

		savePaletteContent("default.pal");

	}
	
	public boolean isCellEmpty(int cell)
	{
		ByteFaker empty = ByteFaker.wrap(new byte[1]);
		
		if( cellhash.get("cell"+cell).isEmpty() || cellhash.get("image"+cell).isEmpty() )
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public void deleteCell(int cell)
	{
		cellContents[cell] = null;
		cellImages[cell] = null;
		
		cellhash.put( "cell"+cell, ByteFaker.wrap(new byte[1]) );
		cellhash.put( "image"+cell, ByteFaker.wrap(new byte[1]) );

		if(isServerActive)
		{
			try
			{		
				sendPacket(makePacket(PaletteCommand.DELETE_CELL.ordinal(), cell));
			}
			catch(Exception e)
			{
				debug("Whatever you did, you broke it.");
				//e.printStackTrace();
			}
		}
	}
	
	private void getCellImage(int cell)
	{
		try
		{
			byte[] rdata = sendRecvPacket(makePacket(PaletteCommand.GET_CELL_IMAGE.ordinal(), cell));
			
			ByteArrayInputStream bais = new ByteArrayInputStream(rdata);
			ObjectInputStream in = new ObjectInputStream(bais);
			cellImages[cell] =  ((ImageIcon)in.readObject()).getImage();
			in.close();
		}
		catch(StreamCorruptedException sce)
		{
			// nobody cares
		}
		catch(Exception e)
		{
			debug("Whatever you did, you broke it.");
			//e.printStackTrace();
		}
	}

	private void getCellContents(int cell)
	{
		try
		{
			byte[] rdata = sendRecvPacket(makePacket(PaletteCommand.GET_CELL_CONTENTS.ordinal(), cell));
			
			ByteArrayInputStream bais = new ByteArrayInputStream(rdata);
			ObjectInputStream in = new ObjectInputStream(bais);
			cellContents[cell] =  new CellContents((HashMap)in.readObject());
			in.close();
		}
		catch(StreamCorruptedException sce)
		{
			// nobody cares
		}
		catch(Exception e)
		{
			debug("Whatever you did, you broke it.");
			//e.printStackTrace();
		}
	}
	private void getCell(int cell)
	{
		if(isServerActive)
		{
			getCellImage(cell);
			getCellContents(cell);
		}
	}
	
	private int getCellWidth()
	{
		return new Double(getBounds().width / col).intValue();
	}
	
	private int getCellHeight()
	{
		return new Double(getBounds().height / row).intValue();
	}
	
	public void setVBCanvas(CalicoCanvas c) {
		canvas = c;
	}
	
	public void paint(PPaintContext context) {
		Graphics2D g2 = context.getGraphics(); 
		//GradientPaint gp = new GradientPaint(75, 75, new Color(219,219,219), 95, 95, Color.gray, true);	
		//g2.setPaint(gp);
		g2.setPaint(getPaint());
		g2.setStroke(new BasicStroke(1.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
		
		int cellWidth = 0;
		int cellHeight = 0;
	
		
		//draw the horizontal lines
		if(isInHiddenMode)
		{
			updateBounds(biggestWindow,200);		
		}
		else
		{
			updateBounds(biggestWindow,200);
			
			cellWidth = new Double( (getBounds().width - nameWidth) / col).intValue();
			cellHeight = new Double(getBounds().height / row).intValue();
			
			double xbounds = getBounds().x + nameWidth;
			double ybounds = getBounds().y;
			
			//draw cell contents
			g2.setPaint(Color.BLACK);
			int counter = 0;
			for (int r = 0; r < row; r++) {
				for (int c = 0; c < col; c++) {
					g2.setPaint(new Color(219,219,219));
					
					g2.fillRect(new Double(xbounds + cellWidth*c).intValue() + 1, 
								new Double(ybounds + cellHeight*r).intValue() + 1, 
								cellWidth, 
								cellHeight
								);
					
					g2.drawImage(	cellImages[counter++], 
									new Double(xbounds + cellWidth*c).intValue() + 2,
									new Double(ybounds + cellHeight*r).intValue() + 2, 
									null
									);
								
					g2.setPaint(Color.BLACK);
					
					g2.drawRect(new Double(xbounds + cellWidth*c).intValue() + 1, 
								new Double(ybounds + cellHeight*r).intValue() + 1, 
								cellWidth,
								cellHeight
								);
				}
			}
			
			for (int i = 1; i < row; i++) {
				g2.drawLine(new Double(xbounds).intValue(), 
							new Double(ybounds + cellHeight*i).intValue(), 
							new Double(xbounds + getBounds().width).intValue(), 
							new Double(ybounds + cellHeight*i).intValue()
							);
			}
			//draw the vertical lines
			for (int i = 1; i < col; i++) {
				g2.drawLine(new Double(xbounds + cellWidth*i).intValue(),
							new Double(ybounds).intValue(), 
							new Double(xbounds + cellWidth*i).intValue(), 
							new Double(ybounds + getBounds().height).intValue()
							);
			}
			
			g2.drawRect(new Double(xbounds).intValue(), 
						new Double(ybounds).intValue(), 
						new Double(getBounds().width).intValue(), 
						new Double(getBounds().height).intValue()
						);
						
						
						
		}
		//draw border
		
		int x = new Double(getBounds().getX()).intValue();
		int y = new Double(getBounds().getY()).intValue();
		int width = new Double(getBounds().getWidth()).intValue();
		int height = new Double(getBounds().getHeight()).intValue();
		
		
		
		g2.setPaint(Color.white);
		g2.fillRect(x , y, nameWidth, height);
		
		g2.setPaint(Color.black);
		g2.drawRect(x, y, nameWidth, height);
		g2.setPaint(Color.black);
		int yo = 1;
		/*
		g2.drawString("P",x+1,(++yo*12)+y);
		g2.drawString("A",x+1,(++yo*12)+y);
		g2.drawString("L",x+1,(++yo*12)+y);
		g2.drawString("E",x+1,(++yo*12)+y);
		g2.drawString("T",x+1,(++yo*12)+y);
		g2.drawString("T",x+1,(++yo*12)+y);
		g2.drawString("E",x+1,(++yo*12)+y);
		g2.drawString("S",x+1,(++yo*12)+y);
		*/
		
		//g2.drawString("Figures", 100, 100);
		
	}
	
	public void showHide()
	{
		isInHiddenMode = !isInHiddenMode;
		repaint();
	}
	public void registerScrap(ArrayList<ScrapPainter> scraps, Point2D p) {
		int cIndex = getIndex(p);
		
		if (cIndex < 0)
			return;
			
			
		canvas.applyAction(new PaletteItemSet() );
		
		
		//copy scrap models
		ArrayList<ScrapModel> tempS = new ArrayList<ScrapModel>();
		for (ScrapPainter sp : scraps) {
			sp.deselect(); 
			tempS.add(ScrapController.copy(sp.getModel()));
			//cellContents[cIndex] = ScrapController.copy(scrap.getModel());
		}
		//determine and copy relationship models (a bit more complicated). Algorithm taken from VBCanvas.deleteScrap
		RelationshipPainter rPainter;
		ArrayList<RelationshipModel> tempR = new ArrayList<RelationshipModel>();
		//remove the relationships attached to the deleted scraps
		for (int i = canvas.getLayer().getChildrenCount() - 1; i >= 0; i--) {
			if (canvas.getLayer().getChild(i) instanceof RelationshipPainter) {
				rPainter = (RelationshipPainter)canvas.getLayer().getChild(i);
				for (int j = 0; j < scraps.size(); j++) 
					if (RelationshipController.connectedTo(
							(rPainter).getModel(), ScrapController.getId(scraps.get(j).getModel()))) {
						//getLayer().removeChild(i);
						tempR.add(rPainter.getModel());
						break;
					}
			}
		}
		//remove relationships who have a hook to a nonexistant scrap
		boolean h1Ver = false, h2Ver = false;
		RelationshipModel rm;
		for (int i = tempR.size()-1; i >= 0; i--) {
			rm = tempR.get(i);
			for (ScrapModel sm : tempS) {
				if (rm.hook1 == sm.id)
					h1Ver = true;
				if (rm.hook2 == sm.id)
					h2Ver= true;
			}
			if (!h1Ver || !h2Ver)
				tempR.remove(i);
			h1Ver = false;
			h2Ver = false;
		}
		
		cellContents[cIndex] = new CellContents(tempS, tempR);
		
		int cellWidth = new Double(getBounds().width / col).intValue();
		int cellHeight = new Double(getBounds().height / row).intValue();
		
		
		PCanvas tempCanvas = new PCanvas();
		ScrapPainter tempPainter;
		for (ScrapPainter sp : scraps) {
			tempPainter = new ScrapPainter(sp.getModel());
			tempPainter.setBounds();
			tempCanvas.getLayer().addChild(tempPainter);
		}
		cellImages[cIndex] = CalicoCanvas.getProportionateScaledImage(tempCanvas.getLayer().toImage(), cellWidth-2, cellHeight-2, Image.SCALE_SMOOTH);
		//cellImages[cIndex] = scrap.toImage().getScaledInstance(cellWidth-2, cellHeight-2, Image.SCALE_SMOOTH);
		
		// YAR
		sendCell(cIndex);
		
		
		repaint();
		
	}
	
	/**
	 * Returns a collection of PNodes, which should be a collection of scraps and relationships. These are copies,
	 * so feel free to abuse the returned array to your leisure.
	 * @param p The point on the table of whom the cell contents you want to return
	 * @param dest The target destination you want your cell contents to be placed at.
	 * @return
	 */
	public CellContents getScrapAt(Point2D p, Point2D dest) {
		
		debug("FB.getScrapAt( " + p + " => "+dest+")");
		
		int index = getIndex(p);
		
		if(index < 0)
		{
			return new CellContents();
		}
		else
		{
			debug("CELL: "+index);
			
			canvas.applyAction(new PaletteItemUsed() );
		
			CellContents temp = cellContents[index];
			//get the absolute center of all the scraps
		
			ArrayList<Point2D> points = new ArrayList<Point2D>();
			for (ScrapModel sm : temp.scraps) {
				points.add(ScrapController.getMidPoint(sm));
			}
			Point2D midPoint = computeMidPoint(points);
		
			CellContents ret = new CellContents(temp);
			int deltaX = new Double(dest.getX() - midPoint.getX()).intValue();
			int deltaY = new Double(dest.getY() - midPoint.getY()).intValue();
			for (ScrapModel sm : ret.scraps)
				ScrapController.translate(sm, deltaX, deltaY);
			for (RelationshipModel rm : ret.rels)
				RelationshipController.translate(rm, deltaX, deltaY);
			
			return ret;
		}
	}
	
	public void movePaletteItem(int index, int index2) {
		if(index < 0 || index2 < 0 )
			return;
		
		if(isCellEmpty(index2))
		{
			cellContents[index2] = cellContents[index];
			cellImages[index2] = cellImages[index];
			
			cellhash.put("cell"+index2, cellhash.get("cell"+index));
			cellhash.put("image"+index2, cellhash.get("image"+index));
			
			deleteCell(index);
			sendCell(index2);
			
			canvas.applyAction(new PaletteItemMoved() );
			
		}
		else
		{
			debug("Cell " + index2 + " is not empty, cannot move to");
		}
		
	}
	
	public Point2D computeMidPoint(ArrayList<Point2D> points) {
		double midX = 0, midY = 0;
		for (Point2D p : points) {
			midX += p.getX();
			midY += p.getY();
		}
		
		midX /= points.size();
		midY /= points.size();
		
		return new Point2D.Double(midX, midY);
	}
	
	public int getIndex(Point2D p)
	{
		return getIndex(p, isInHiddenMode);
	}
	/**
	 * Returns the numerical index of the cell that the point occupies. Returns -1 if the index is out of bounds.
	 * 
	 * @param p
	 * @return 
	 */
	public int getIndex(Point2D p,boolean isOuterMove) {
		
		debug("Cell Click Point: " + p);
		int cellWidth = new Double( (getBounds().width-nameWidth) / col).intValue();
		int cellHeight = new Double(getBounds().height / row).intValue();
		
		//alg: create a rect, check if it's in it, then move on to next rect.
		Rectangle2D rect;
		//check if it's the handle, and if it is, return the constant
		rect = new Rectangle2D.Double( getBounds().x, getBounds().y, nameWidth, getBounds().height);
		if (rect.contains(p))
			return PULL_HANDLE;
		
		for (int r = 0; r < row; r++) {
			for (int c = 0; c < col; c++) {
				rect = new Rectangle2D.Double(	(getBounds().x+nameWidth) + cellWidth*c,
												getBounds().y + cellHeight*r, 
												cellWidth,
												cellHeight
												);
				if (rect.contains(p))
				{
					//System.out.println("Cell Index: " + (r*col + c) );
					return r*col + c;
				}
			}
		}
		
		/*
		//Nick: So this code right seems really random from my point of view, and is causing problems.
		if(!isOuterMove)
		{
			if(!isInHiddenMode)
			{
				showHide();
			}
		}
		else if(isInHiddenMode)
		{
			showHide();
		}
		*/
		
		return -1;
	}
	
	public boolean contains(Point2D p) {
		debug("CONTAINS? " + p + " = " + getBounds().contains(p) );
		return getBounds().contains(p);
			
	}
	
	
	@Override
	public void registerCanvas(CalicoCanvas c) {
		canvas = c;

		canvas.addCalicoActionListener(this);
	}
	public void fireCalicoListener(CalicoActionEvent vbe) {
	/*	
		if (vbe.getAction() instanceof UndoAction) {
			if (((UndoAction)vbe.getAction()).type == UndoAction.UNDO)
				undoIndex++;
			else if (((UndoAction)vbe.getAction()).type == UndoAction.REDO)
				undoIndex--;
			else
				System.err.println("Undo:fireCalicoListener - bad UndoAction type");
		}
		else {
			//if the user clicks undo several times, then performs a new action, then
			//we remove the possibility to redo.
			if (undoIndex > 0) {
				for (int i = 0; i < undoIndex; i++)
					modelHistory.remove(0);
			}
			undoIndex = 0;
			modelHistory.add(0, vbe.getEventSource().getModel());
			
			if (modelHistory.size() > maxUndo)
				modelHistory.remove(modelHistory.size() - 1);
		}*/
	}

	@Override
	public boolean updateBounds(int width, int height) {
		return updateBounds(width,height,50+nameWidth);
	}
	
	public boolean updateBounds(int width, int height, int realWidth)
	{
		if( biggestWindow < width )
		{
			biggestWindow = width;
		}
		
		int offset = new Double(nameWidth).intValue();
		if(!isInHiddenMode)
		{
			offset = 50 + offset;
		}
		
		//if (setBounds(width-offset, 200, realWidth, 125))
		if (setBounds(width-offset, 100, realWidth, 225))
			return true;
		return false;
	}


	
	private class PalettesListener extends PBasicInputEventHandler {
		
		public void mouseReleased(PInputEvent e)
		{
			System.out.println("MOUSE RELEASED");
		}
		public void mousePressed(PInputEvent e) {
			System.out.println("MOUSE Pressed");
			
			

		}
		
	}


}
