package ics.uci.edu.Calico.classroom;


import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.models.ObjectHandlerModel;
import ics.uci.edu.Calico.models.actions.CalicoAction;
import ics.uci.edu.Calico.models.actions.CalicoActionEvent;
import ics.uci.edu.Calico.models.actions.CalicoActionListener;
import ics.uci.edu.Calico.networking.ClientInputListener;
import ics.uci.edu.Calico.networking.NetworkPacket;

import javax.swing.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.*;
import java.net.*;

public class ClassroomSocketClient{

	private static final long serialVersionUID = 1L;
	private Socket socket;
	private ObjectOutputStream out;
	private ObjectInputStream in;
	private boolean connected = false;
	private boolean inError = false;
	private NetworkPacket networkPacket;
	private CalicoCanvas canvas;
	private ClassroomClientUI clientUI;
	private ObjectHandlerModel premadeCanvas;

  
  ClassroomSocketClient(CalicoCanvas canvas, ClassroomClientUI cUI){ 
      socket = null;
      out = null;
      in = null;
      networkPacket = null;
      this.canvas = canvas;
      clientUI = cUI;
      connected = false;
      }
      
   public boolean isConnected(){
	   return connected;
   }
   
   public void sendCurrentCanvas(){
	   try {
			NetworkPacket nP = new NetworkPacket(canvas.getModel());
			out.writeObject(nP);
	 	} catch (IOException e) {
			connected = false;
		}
   }
   
   public void sendCanvas(ObjectHandlerModel oHM){
	   try {
			NetworkPacket nP = new NetworkPacket(oHM);
			out.writeObject(nP);
	 	} catch (IOException e) {
			connected = false;
		}
   }
   
   //when action occurs write action to output stream
   public void fireAction(CalicoActionEvent action){
	   if(isConnected()){
		   networkPacket = new NetworkPacket(action.getAction());
		          try {
		        	  System.out.println("A");
		        	  out.writeObject(networkPacket);
				} catch (IOException e) {
					message("Warning: Server not active");
					connected = false;
					finalize();
					clientUI.drawConnectionFrame();
				}
	   	}
   }
  
  
  public void connect(String serverAddress, String nickname){
	  try {
				socket = new Socket(serverAddress, 6112);
				connected = true;
				message("Connected!");
				clientUI.removeConnectFrame();
				inError = false;
			} catch (UnknownHostException e) {
				message("Could not find server");
				connected = false;
				inError = true;
			} catch (IOException e) {
				message("Could not find server");
				connected = false;
				inError = true;
			}
			
	//if not in error	
			if(!inError){
    	 	try {
				out = new ObjectOutputStream(socket.getOutputStream());
				in = new ObjectInputStream(socket.getInputStream());
				sendCurrentCanvas();
				NetworkPacket nP2 = new NetworkPacket(nickname);
				out.writeObject(nP2);
				
				ClassClientInputListener c = new ClassClientInputListener(in,this);
	    		Thread t = new Thread(c);
	    		t.start();
				
    	 	} catch (IOException e) {
				connected = false;
				e.printStackTrace();
			}
    	 
	}
			
 }
  
  //this is what happens when an input is received 
  public void performAction(CalicoAction action){
	  
	  CalicoActionListener actionListener = null;
	  for (CalicoActionListener listener : canvas.getListeners())
		  if (listener instanceof ClassroomClientUI)
			  actionListener = listener;
	  canvas.removeCalicoActionListener(actionListener);
	  canvas.applyAction(action);
	  canvas.repaint();
	  canvas.addCalicoActionListener(actionListener);
  }
  

  
  
  public void setTimer(){
	  System.out.println("TIMER STARTED");
	   int delay = 5000; //milliseconds(1000ms = 1s)
	   ActionListener taskPerformer = new ActionListener() {
	       public void actionPerformed(ActionEvent evt){ 
	    	   sendCurrentCanvas();
	       }
	   };
	   new Timer(delay, taskPerformer).start();
  }
  
  public void setCanvas(ObjectHandlerModel pC){
	  premadeCanvas = pC;
	  clientUI.warningMessage();
  }
 
  public void saveCurrentCanvas(){
	  save();
	  message("Canvas Loaded!!");
	  canvas.loadModel(premadeCanvas);
	  canvas.repaint();
  }
  
  public void dontSaveCurrentCanvas(){
	  message("Canvas Loaded!!");
	  canvas.loadModel(premadeCanvas);
	  canvas.repaint();
  }
  
  public void save(){
	  clientUI.getMenu().saveCanvas();
  }
 
  
  
  protected void finalize(){
	 
	    connected = false; 
	  	try{
	         out.close();
	         in.close();
	         message("Disconnected!");
	    } catch (IOException e) {
	        System.out.println("Could not close.");
	    }
	  }
  
 
   private void message(String message){
	   JFrame errorFrame = new JFrame();
	   JOptionPane.showMessageDialog(errorFrame,
			    message,
			    "Client Message",
			    JOptionPane.INFORMATION_MESSAGE);
	   
   }
   
}
