package ics.uci.edu.Calico.networking;


import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.UI.components.CalicoGrid;
import ics.uci.edu.Calico.UI.components.Menu;
import ics.uci.edu.Calico.models.actions.CalicoActionEvent;
import ics.uci.edu.Calico.models.actions.CalicoActionListener;
import ics.uci.edu.Calico.models.actions.ChangedCell;
import ics.uci.edu.Calico.models.actions.GridView;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class Networking extends JFrame implements CalicoActionListener {
	
		private static final long serialVersionUID = 1L;
		private SocketClient client;
		private JFrame connectionFrame;
		private JTextField serverAddressBox;
		private JTextField nicknameBox;
		private JButton connectButton;
		private String serverAddress;
		private String nickname;
		protected Menu menu;
		protected CalicoGrid grid;
		private boolean clientStarted;
		protected CalicoCanvas canvas;
		private ArrayList<Integer> activatedCanvasIDs;
		private boolean flag;
		
		public Networking(){
			canvas = new CalicoCanvas();
			flag = false;
			serverAddress = "localhost";
			nickname = "";
			clientStarted = false;
			activatedCanvasIDs = new ArrayList<Integer>();
		}
		
		public void startServer(){
			SocketMultiThreadedServer server = new SocketMultiThreadedServer(null);
			Thread t = new Thread(server);
			t.start();
		}
		
		public void startClient(){
			client = new SocketClient(this);
			clientStarted = true;
		}
		
		public void requestGridUpdate(){
			if(clientStarted){
				char temp = 'a';
				client.sendNetworkPacket(new NetworkPacket(temp));
				System.out.println("CLIENT: request grid update!");
			}
		}
		
		/**
		 * Activates a canvas by 
		 * @param canvas the canvas being activated
		 * @param ID The ID(position) of that canvas
		 */
		public void activateClientCanvas(Integer ID){
			  
			
			if(!activatedCanvasIDs.contains(ID)){
				//this.canvas.addCalicoActionListener(this);
				activatedCanvasIDs.add(ID);
				
				if(clientStarted){
					System.out.println("Client: Activate Canvas " + ID);
					client.setCanvas(ID);
					requestCanvasActivation(ID);
					}
			}
		}
		
		public void requestCanvasActivation(int ID){
			client.sendNetworkPacket(new NetworkPacket(ID,nickname));
		}
		
		public void setCurrentCalicoCanvas(CalicoCanvas canvas){
			CalicoActionListener actionListener = null;
			
			  for (CalicoActionListener listener : this.canvas.getListeners())
				  if (listener instanceof Networking)
					  actionListener = listener;
			  canvas.removeCalicoActionListener(actionListener);
			  canvas.repaint();
			  this.canvas = canvas;
			  this.canvas.addCalicoActionListener(this);
		}
		
		public void fireCalicoListener(CalicoActionEvent action){
			if(clientStarted){
				 if (action.getAction() instanceof GridView && clientStarted){
					 char temp = 'a';
					 client.sendNetworkPacket(new NetworkPacket(temp));
					 System.out.println("CLIENT: request grid update!");
				 }
				 else if(action.getAction() instanceof ChangedCell){
					 //NOTHING YET!!! TODO
				 }
				 else{
					System.out.println("firing action: " + action.getAction());
					client.fireAction(action, nickname);
				 }
					 
			}
			
		}
		
		protected void drawMenu(){
		
		}
		
		public void drawConnectionFrame() {
			GridBagConstraints d = new GridBagConstraints();
			connectionFrame = new JFrame();
			
			JPanel ipAddress = new JPanel();
			JPanel nicknamePanel = new JPanel();
			
			JLabel ipAddressLabel = new JLabel("IP Address: ");
			JLabel nicknameLabel = new JLabel("Nickname:  ");
			
			
			connectionFrame.setTitle("Connect");
			connectionFrame.setLocation(new Point(225,250));
			connectionFrame.setLayout(new GridBagLayout());
			ipAddress.setLayout(new GridBagLayout());
			nicknamePanel.setLayout(new GridBagLayout());
			
			serverAddressBox = new JTextField(1);
	   		connectButton = new JButton();
	   		nicknameBox = new JTextField(1);
	   		
	   		connectionFrame.setPreferredSize(new Dimension(250,150));
	   		
			connectButton.setText("Connect");
	        connectButton.addActionListener(new java.awt.event.ActionListener() {
	            public void actionPerformed(java.awt.event.ActionEvent evt) {
	                connect(evt);
	            }
	        });
	        
	        
	        serverAddressBox.setText(serverAddress);
	        serverAddressBox.addActionListener(new java.awt.event.ActionListener() {
	            public void actionPerformed(java.awt.event.ActionEvent evt) {
	                connect(evt);
	            }
	        });
	        
	        nicknameBox.setText(nickname);
	        nicknameBox.addActionListener(new java.awt.event.ActionListener() {
	            public void actionPerformed(java.awt.event.ActionEvent evt) {
	            	connect(evt);
	            }
	        });
	        
	        nicknameBox.repaint();
	        nicknamePanel.add(nicknameLabel);
	        d.ipadx = 100;
	        d.insets = new Insets(5,20,5,5);
	        nicknamePanel.add(nicknameBox,d);
	        d.gridy = 0;
	        connectionFrame.add(nicknamePanel,d);
	     
	        ipAddress.add(ipAddressLabel);
	        
	        d.ipadx = 100;
	        d.insets = new Insets(5,20,5,5);
	        ipAddress.add(serverAddressBox,d);
	        d.gridy=1;
	        connectionFrame.add(ipAddress,d);
	     
			d.gridy = 2;
			d.ipadx = 100;
			connectionFrame.add(connectButton,d);

			connectionFrame.setAlwaysOnTop(true);
			connectionFrame.setVisible(true);
			connectionFrame.repaint();
			connectionFrame.pack();
		}
		
		

		public void connect(java.awt.event.ActionEvent evt) {
			   if(client.isConnected()){
				 message("Already Connected!");
			   }
			   else{
				   serverAddress = serverAddressBox.getText();
				   nickname = nicknameBox.getText();
				   if(nickname.equals("")){
					  message("Please enter a nickname to use");
				   }else{
					   client.connect(serverAddress,nickname);
					   this.setTitle(nicknameBox.getText() + "'s Calico Client");
				   }
			   }   
		   }
		
		public void setCalicoGrid(CalicoGrid grid){
			this.grid = grid;
		}
		
		public CalicoGrid getCalicoGrid(){
			return grid;
		}
		
		public void removeConnectFrame(){
			connectionFrame.dispose();
		}
		
		public Menu getMenu(){
			return menu;
		}
		
		 private void message(String message){
			   JFrame errorFrame = new JFrame();
			   JOptionPane.showMessageDialog(errorFrame,
					    message,
					    "Client Message",
					    JOptionPane.INFORMATION_MESSAGE);
			   
		   }
		 
		  
		  public void disconnect(){
			  client.finalize();
		  }
		 
		  public static void main(String[] args){
		        Networking frame = new Networking();
		        WindowListener l = new WindowAdapter() {
		                public void windowClosing(WindowEvent e) {
		                	System.exit(0);
		                }
		        };
		        //frame.connect(null);
		        frame.addWindowListener(l);
		        frame.pack();
		        frame.setVisible(true);
		  }
		  
		  //TEST
		 
	
	
	
}
